# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class ExportCustomJobList(ListResource):

    def __init__(self, version, resource_type):
        """
        Initialize the ExportCustomJobList

        :param Version version: Version that contains the resource
        :param resource_type: The type of communication – Messages, Calls, Conferences, and Participants

        :returns: twilio.rest.bulkexports.v1.export.export_custom_job.ExportCustomJobList
        :rtype: twilio.rest.bulkexports.v1.export.export_custom_job.ExportCustomJobList
        """
        super(ExportCustomJobList, self).__init__(version)

        # Path Solution
        self._solution = {'resource_type': resource_type, }
        self._uri = '/Exports/{resource_type}/Jobs'.format(**self._solution)

    def stream(self, limit=None, page_size=None):
        """
        Streams ExportCustomJobInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.bulkexports.v1.export.export_custom_job.ExportCustomJobInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, limit=None, page_size=None):
        """
        Lists ExportCustomJobInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.bulkexports.v1.export.export_custom_job.ExportCustomJobInstance]
        """
        return list(self.stream(limit=limit, page_size=page_size, ))

    def page(self, page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of ExportCustomJobInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of ExportCustomJobInstance
        :rtype: twilio.rest.bulkexports.v1.export.export_custom_job.ExportCustomJobPage
        """
        data = values.of({'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return ExportCustomJobPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of ExportCustomJobInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of ExportCustomJobInstance
        :rtype: twilio.rest.bulkexports.v1.export.export_custom_job.ExportCustomJobPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return ExportCustomJobPage(self._version, response, self._solution)

    def create(self, start_day, end_day, friendly_name, webhook_url=values.unset,
               webhook_method=values.unset, email=values.unset):
        """
        Create the ExportCustomJobInstance

        :param unicode start_day: The start day for the custom export specified as a string in the format of yyyy-mm-dd
        :param unicode end_day: The end day for the custom export specified as a string in the format of yyyy-mm-dd. End day is inclusive and must be 2 days earlier than the current UTC day.
        :param unicode friendly_name: The friendly name specified when creating the job
        :param unicode webhook_url: The optional webhook url called on completion of the job. If this is supplied, `WebhookMethod` must also be supplied.
        :param unicode webhook_method: This is the method used to call the webhook on completion of the job. If this is supplied, `WebhookUrl` must also be supplied.
        :param unicode email: The optional email to send the completion notification to

        :returns: The created ExportCustomJobInstance
        :rtype: twilio.rest.bulkexports.v1.export.export_custom_job.ExportCustomJobInstance
        """
        data = values.of({
            'StartDay': start_day,
            'EndDay': end_day,
            'FriendlyName': friendly_name,
            'WebhookUrl': webhook_url,
            'WebhookMethod': webhook_method,
            'Email': email,
        })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return ExportCustomJobInstance(
            self._version,
            payload,
            resource_type=self._solution['resource_type'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Bulkexports.V1.ExportCustomJobList>'


class ExportCustomJobPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the ExportCustomJobPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param resource_type: The type of communication – Messages, Calls, Conferences, and Participants

        :returns: twilio.rest.bulkexports.v1.export.export_custom_job.ExportCustomJobPage
        :rtype: twilio.rest.bulkexports.v1.export.export_custom_job.ExportCustomJobPage
        """
        super(ExportCustomJobPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of ExportCustomJobInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.bulkexports.v1.export.export_custom_job.ExportCustomJobInstance
        :rtype: twilio.rest.bulkexports.v1.export.export_custom_job.ExportCustomJobInstance
        """
        return ExportCustomJobInstance(
            self._version,
            payload,
            resource_type=self._solution['resource_type'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Bulkexports.V1.ExportCustomJobPage>'


class ExportCustomJobInstance(InstanceResource):

    class Status(object):
        ERRORDURINGRUN = "ErrorDuringRun"
        SUBMITTED = "Submitted"
        RUNNING = "Running"
        COMPLETEDEMPTYRECORDS = "CompletedEmptyRecords"
        COMPLETED = "Completed"
        FAILED = "Failed"
        RUNNINGTOBEDELETED = "RunningToBeDeleted"
        DELETEDBYUSERREQUEST = "DeletedByUserRequest"

    def __init__(self, version, payload, resource_type):
        """
        Initialize the ExportCustomJobInstance

        :returns: twilio.rest.bulkexports.v1.export.export_custom_job.ExportCustomJobInstance
        :rtype: twilio.rest.bulkexports.v1.export.export_custom_job.ExportCustomJobInstance
        """
        super(ExportCustomJobInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'friendly_name': payload.get('friendly_name'),
            'resource_type': payload.get('resource_type'),
            'start_day': payload.get('start_day'),
            'end_day': payload.get('end_day'),
            'webhook_url': payload.get('webhook_url'),
            'webhook_method': payload.get('webhook_method'),
            'email': payload.get('email'),
            'job_sid': payload.get('job_sid'),
            'details': payload.get('details'),
            'job_queue_position': payload.get('job_queue_position'),
            'estimated_completion_time': payload.get('estimated_completion_time'),
        }

        # Context
        self._context = None
        self._solution = {'resource_type': resource_type, }

    @property
    def friendly_name(self):
        """
        :returns: The friendly name specified when creating the job
        :rtype: unicode
        """
        return self._properties['friendly_name']

    @property
    def resource_type(self):
        """
        :returns: The type of communication – Messages, Calls, Conferences, and Participants
        :rtype: unicode
        """
        return self._properties['resource_type']

    @property
    def start_day(self):
        """
        :returns: The start day for the custom export specified as a string in the format of yyyy-MM-dd
        :rtype: unicode
        """
        return self._properties['start_day']

    @property
    def end_day(self):
        """
        :returns: The end day for the custom export specified as a string in the format of yyyy-MM-dd. This will be the last day exported. For instance, to export a single day, choose the same day for start and end day. To export the first 4 days of July, you would set the start date to 2020-07-01 and the end date to 2020-07-04. The end date must be the UTC day before yesterday.
        :rtype: unicode
        """
        return self._properties['end_day']

    @property
    def webhook_url(self):
        """
        :returns: The optional webhook url called on completion
        :rtype: unicode
        """
        return self._properties['webhook_url']

    @property
    def webhook_method(self):
        """
        :returns: This is the method used to call the webhook
        :rtype: unicode
        """
        return self._properties['webhook_method']

    @property
    def email(self):
        """
        :returns: The optional email to send the completion notification to
        :rtype: unicode
        """
        return self._properties['email']

    @property
    def job_sid(self):
        """
        :returns: The unique job_sid returned when the custom export was created. This can be used to look up the status of the job.
        :rtype: unicode
        """
        return self._properties['job_sid']

    @property
    def details(self):
        """
        :returns: The details of a job state which is an object that contains a `status` string, a day count integer, and list of days in the job
        :rtype: dict
        """
        return self._properties['details']

    @property
    def job_queue_position(self):
        """
        :returns: This is the job position from the 1st in line. Your queue position will never increase. As jobs ahead of yours in the queue are processed, the queue position number will decrease
        :rtype: unicode
        """
        return self._properties['job_queue_position']

    @property
    def estimated_completion_time(self):
        """
        :returns: this is the time estimated until your job is complete. This is calculated each time you request the job list. The time is calculated based on the current rate of job completion (which may vary) and your job queue position
        :rtype: unicode
        """
        return self._properties['estimated_completion_time']

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Bulkexports.V1.ExportCustomJobInstance>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class TokenList(ListResource):

    def __init__(self, version):
        """
        Initialize the TokenList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.oauth.v1.token.TokenList
        :rtype: twilio.rest.oauth.v1.token.TokenList
        """
        super(TokenList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/token'.format(**self._solution)

    def create(self, grant_type, client_sid, client_secret=values.unset,
               code=values.unset, code_verifier=values.unset,
               device_code=values.unset, refresh_token=values.unset,
               device_id=values.unset):
        """
        Create the TokenInstance

        :param unicode grant_type: A way of representing resource owner's to obtain access token
        :param unicode client_sid: A string that uniquely identifies this oauth app
        :param unicode client_secret: The credential for confidential OAuth App
        :param unicode code: Jwt token
        :param unicode code_verifier: The cryptographically generated code
        :param unicode device_code: Jwt token
        :param unicode refresh_token: Jwt token
        :param unicode device_id: An Id of device

        :returns: The created TokenInstance
        :rtype: twilio.rest.oauth.v1.token.TokenInstance
        """
        data = values.of({
            'GrantType': grant_type,
            'ClientSid': client_sid,
            'ClientSecret': client_secret,
            'Code': code,
            'CodeVerifier': code_verifier,
            'DeviceCode': device_code,
            'RefreshToken': refresh_token,
            'DeviceId': device_id,
        })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return TokenInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Oauth.V1.TokenList>'


class TokenPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the TokenPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.oauth.v1.token.TokenPage
        :rtype: twilio.rest.oauth.v1.token.TokenPage
        """
        super(TokenPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of TokenInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.oauth.v1.token.TokenInstance
        :rtype: twilio.rest.oauth.v1.token.TokenInstance
        """
        return TokenInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Oauth.V1.TokenPage>'


class TokenInstance(InstanceResource):

    def __init__(self, version, payload):
        """
        Initialize the TokenInstance

        :returns: twilio.rest.oauth.v1.token.TokenInstance
        :rtype: twilio.rest.oauth.v1.token.TokenInstance
        """
        super(TokenInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'access_token': payload.get('access_token'),
            'refresh_token': payload.get('refresh_token'),
            'id_token': payload.get('id_token'),
            'refresh_token_expires_at': deserialize.iso8601_datetime(payload.get('refresh_token_expires_at')),
            'access_token_expires_at': deserialize.iso8601_datetime(payload.get('access_token_expires_at')),
        }

        # Context
        self._context = None
        self._solution = {}

    @property
    def access_token(self):
        """
        :returns: Token which carries the necessary information to access a Twilio resource directly
        :rtype: unicode
        """
        return self._properties['access_token']

    @property
    def refresh_token(self):
        """
        :returns: Token which carries the information necessary to get a new access token
        :rtype: unicode
        """
        return self._properties['refresh_token']

    @property
    def id_token(self):
        """
        :returns: The id_token
        :rtype: unicode
        """
        return self._properties['id_token']

    @property
    def refresh_token_expires_at(self):
        """
        :returns: The RFC 2822 date and time in GMT when the refresh token expires
        :rtype: datetime
        """
        return self._properties['refresh_token_expires_at']

    @property
    def access_token_expires_at(self):
        """
        :returns: The RFC 2822 date and time in GMT when the access token expires
        :rtype: datetime
        """
        return self._properties['access_token_expires_at']

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Oauth.V1.TokenInstance>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class GoodDataList(ListResource):

    def __init__(self, version):
        """
        Initialize the GoodDataList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.flex_api.v1.good_data.GoodDataList
        :rtype: twilio.rest.flex_api.v1.good_data.GoodDataList
        """
        super(GoodDataList, self).__init__(version)

        # Path Solution
        self._solution = {}

    def get(self):
        """
        Constructs a GoodDataContext

        :returns: twilio.rest.flex_api.v1.good_data.GoodDataContext
        :rtype: twilio.rest.flex_api.v1.good_data.GoodDataContext
        """
        return GoodDataContext(self._version, )

    def __call__(self):
        """
        Constructs a GoodDataContext

        :returns: twilio.rest.flex_api.v1.good_data.GoodDataContext
        :rtype: twilio.rest.flex_api.v1.good_data.GoodDataContext
        """
        return GoodDataContext(self._version, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V1.GoodDataList>'


class GoodDataPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the GoodDataPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.flex_api.v1.good_data.GoodDataPage
        :rtype: twilio.rest.flex_api.v1.good_data.GoodDataPage
        """
        super(GoodDataPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of GoodDataInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.flex_api.v1.good_data.GoodDataInstance
        :rtype: twilio.rest.flex_api.v1.good_data.GoodDataInstance
        """
        return GoodDataInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V1.GoodDataPage>'


class GoodDataContext(InstanceContext):

    def __init__(self, version):
        """
        Initialize the GoodDataContext

        :param Version version: Version that contains the resource

        :returns: twilio.rest.flex_api.v1.good_data.GoodDataContext
        :rtype: twilio.rest.flex_api.v1.good_data.GoodDataContext
        """
        super(GoodDataContext, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/Accounts/GoodData'.format(**self._solution)

    def create(self, token=values.unset):
        """
        Create the GoodDataInstance

        :param unicode token: The Token HTTP request header

        :returns: The created GoodDataInstance
        :rtype: twilio.rest.flex_api.v1.good_data.GoodDataInstance
        """
        headers = values.of({'Token': token, })

        payload = self._version.create(method='POST', uri=self._uri, headers=headers, )

        return GoodDataInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.FlexApi.V1.GoodDataContext {}>'.format(context)


class GoodDataInstance(InstanceResource):

    def __init__(self, version, payload):
        """
        Initialize the GoodDataInstance

        :returns: twilio.rest.flex_api.v1.good_data.GoodDataInstance
        :rtype: twilio.rest.flex_api.v1.good_data.GoodDataInstance
        """
        super(GoodDataInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'workspace_id': payload.get('workspace_id'),
            'session_expiry': payload.get('session_expiry'),
            'session_id': payload.get('session_id'),
            'gd_base_url': payload.get('gd_base_url'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {}

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: GoodDataContext for this GoodDataInstance
        :rtype: twilio.rest.flex_api.v1.good_data.GoodDataContext
        """
        if self._context is None:
            self._context = GoodDataContext(self._version, )
        return self._context

    @property
    def workspace_id(self):
        """
        :returns: Unique workspace ID in gooddata
        :rtype: unicode
        """
        return self._properties['workspace_id']

    @property
    def session_expiry(self):
        """
        :returns: The session expiry date and time
        :rtype: unicode
        """
        return self._properties['session_expiry']

    @property
    def session_id(self):
        """
        :returns: Unique session ID
        :rtype: unicode
        """
        return self._properties['session_id']

    @property
    def gd_base_url(self):
        """
        :returns: GoodData login base URL
        :rtype: unicode
        """
        return self._properties['gd_base_url']

    @property
    def url(self):
        """
        :returns: The URL of this resource.
        :rtype: unicode
        """
        return self._properties['url']

    def create(self, token=values.unset):
        """
        Create the GoodDataInstance

        :param unicode token: The Token HTTP request header

        :returns: The created GoodDataInstance
        :rtype: twilio.rest.flex_api.v1.good_data.GoodDataInstance
        """
        return self._proxy.create(token=token, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.FlexApi.V1.GoodDataInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class ChannelList(ListResource):

    def __init__(self, version):
        """
        Initialize the ChannelList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.chat.v3.channel.ChannelList
        :rtype: twilio.rest.chat.v3.channel.ChannelList
        """
        super(ChannelList, self).__init__(version)

        # Path Solution
        self._solution = {}

    def get(self, service_sid, sid):
        """
        Constructs a ChannelContext

        :param service_sid: Service Sid.
        :param sid: A string that uniquely identifies this Channel.

        :returns: twilio.rest.chat.v3.channel.ChannelContext
        :rtype: twilio.rest.chat.v3.channel.ChannelContext
        """
        return ChannelContext(self._version, service_sid=service_sid, sid=sid, )

    def __call__(self, service_sid, sid):
        """
        Constructs a ChannelContext

        :param service_sid: Service Sid.
        :param sid: A string that uniquely identifies this Channel.

        :returns: twilio.rest.chat.v3.channel.ChannelContext
        :rtype: twilio.rest.chat.v3.channel.ChannelContext
        """
        return ChannelContext(self._version, service_sid=service_sid, sid=sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Chat.V3.ChannelList>'


class ChannelPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the ChannelPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.chat.v3.channel.ChannelPage
        :rtype: twilio.rest.chat.v3.channel.ChannelPage
        """
        super(ChannelPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of ChannelInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.chat.v3.channel.ChannelInstance
        :rtype: twilio.rest.chat.v3.channel.ChannelInstance
        """
        return ChannelInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Chat.V3.ChannelPage>'


class ChannelContext(InstanceContext):

    def __init__(self, version, service_sid, sid):
        """
        Initialize the ChannelContext

        :param Version version: Version that contains the resource
        :param service_sid: Service Sid.
        :param sid: A string that uniquely identifies this Channel.

        :returns: twilio.rest.chat.v3.channel.ChannelContext
        :rtype: twilio.rest.chat.v3.channel.ChannelContext
        """
        super(ChannelContext, self).__init__(version)

        # Path Solution
        self._solution = {'service_sid': service_sid, 'sid': sid, }
        self._uri = '/Services/{service_sid}/Channels/{sid}'.format(**self._solution)

    def update(self, type=values.unset, messaging_service_sid=values.unset,
               x_twilio_webhook_enabled=values.unset):
        """
        Update the ChannelInstance

        :param ChannelInstance.ChannelType type: The Type for this Channel to migrate to.
        :param unicode messaging_service_sid: The unique ID of the Messaging Service this channel belongs to.
        :param ChannelInstance.WebhookEnabledType x_twilio_webhook_enabled: The X-Twilio-Webhook-Enabled HTTP request header

        :returns: The updated ChannelInstance
        :rtype: twilio.rest.chat.v3.channel.ChannelInstance
        """
        data = values.of({'Type': type, 'MessagingServiceSid': messaging_service_sid, })
        headers = values.of({'X-Twilio-Webhook-Enabled': x_twilio_webhook_enabled, })

        payload = self._version.update(method='POST', uri=self._uri, data=data, headers=headers, )

        return ChannelInstance(
            self._version,
            payload,
            service_sid=self._solution['service_sid'],
            sid=self._solution['sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Chat.V3.ChannelContext {}>'.format(context)


class ChannelInstance(InstanceResource):

    class ChannelType(object):
        PUBLIC = "public"
        PRIVATE = "private"

    class WebhookEnabledType(object):
        TRUE = "true"
        FALSE = "false"

    def __init__(self, version, payload, service_sid=None, sid=None):
        """
        Initialize the ChannelInstance

        :returns: twilio.rest.chat.v3.channel.ChannelInstance
        :rtype: twilio.rest.chat.v3.channel.ChannelInstance
        """
        super(ChannelInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'account_sid': payload.get('account_sid'),
            'service_sid': payload.get('service_sid'),
            'friendly_name': payload.get('friendly_name'),
            'unique_name': payload.get('unique_name'),
            'attributes': payload.get('attributes'),
            'type': payload.get('type'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'created_by': payload.get('created_by'),
            'members_count': deserialize.integer(payload.get('members_count')),
            'messages_count': deserialize.integer(payload.get('messages_count')),
            'messaging_service_sid': payload.get('messaging_service_sid'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {
            'service_sid': service_sid or self._properties['service_sid'],
            'sid': sid or self._properties['sid'],
        }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: ChannelContext for this ChannelInstance
        :rtype: twilio.rest.chat.v3.channel.ChannelContext
        """
        if self._context is None:
            self._context = ChannelContext(
                self._version,
                service_sid=self._solution['service_sid'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the resource
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def service_sid(self):
        """
        :returns: The SID of the Service that the resource is associated with
        :rtype: unicode
        """
        return self._properties['service_sid']

    @property
    def friendly_name(self):
        """
        :returns: The string that you assigned to describe the resource
        :rtype: unicode
        """
        return self._properties['friendly_name']

    @property
    def unique_name(self):
        """
        :returns: An application-defined string that uniquely identifies the resource
        :rtype: unicode
        """
        return self._properties['unique_name']

    @property
    def attributes(self):
        """
        :returns: The JSON string that stores application-specific data
        :rtype: unicode
        """
        return self._properties['attributes']

    @property
    def type(self):
        """
        :returns: The visibility of the channel. Can be: `public` or `private`
        :rtype: ChannelInstance.ChannelType
        """
        return self._properties['type']

    @property
    def date_created(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was last updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def created_by(self):
        """
        :returns: The identity of the User that created the channel
        :rtype: unicode
        """
        return self._properties['created_by']

    @property
    def members_count(self):
        """
        :returns: The number of Members in the Channel
        :rtype: unicode
        """
        return self._properties['members_count']

    @property
    def messages_count(self):
        """
        :returns: The number of Messages that have been passed in the Channel
        :rtype: unicode
        """
        return self._properties['messages_count']

    @property
    def messaging_service_sid(self):
        """
        :returns: The unique ID of the Messaging Service this channel belongs to.
        :rtype: unicode
        """
        return self._properties['messaging_service_sid']

    @property
    def url(self):
        """
        :returns: The absolute URL of the Channel resource
        :rtype: unicode
        """
        return self._properties['url']

    def update(self, type=values.unset, messaging_service_sid=values.unset,
               x_twilio_webhook_enabled=values.unset):
        """
        Update the ChannelInstance

        :param ChannelInstance.ChannelType type: The Type for this Channel to migrate to.
        :param unicode messaging_service_sid: The unique ID of the Messaging Service this channel belongs to.
        :param ChannelInstance.WebhookEnabledType x_twilio_webhook_enabled: The X-Twilio-Webhook-Enabled HTTP request header

        :returns: The updated ChannelInstance
        :rtype: twilio.rest.chat.v3.channel.ChannelInstance
        """
        return self._proxy.update(
            type=type,
            messaging_service_sid=messaging_service_sid,
            x_twilio_webhook_enabled=x_twilio_webhook_enabled,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Chat.V3.ChannelInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.domain import Domain
from twilio.rest.voice.v1 import V1


class Voice(Domain):

    def __init__(self, twilio):
        """
        Initialize the Voice Domain

        :returns: Domain for Voice
        :rtype: twilio.rest.voice.Voice
        """
        super(Voice, self).__init__(twilio)

        self.base_url = 'https://voice.twilio.com'

        # Versions
        self._v1 = None

    @property
    def v1(self):
        """
        :returns: Version v1 of voice
        :rtype: twilio.rest.voice.v1.V1
        """
        if self._v1 is None:
            self._v1 = V1(self)
        return self._v1

    @property
    def archived_calls(self):
        """
        :rtype: twilio.rest.voice.v1.archived_call.ArchivedCallList
        """
        return self.v1.archived_calls

    @property
    def byoc_trunks(self):
        """
        :rtype: twilio.rest.voice.v1.byoc_trunk.ByocTrunkList
        """
        return self.v1.byoc_trunks

    @property
    def connection_policies(self):
        """
        :rtype: twilio.rest.voice.v1.connection_policy.ConnectionPolicyList
        """
        return self.v1.connection_policies

    @property
    def dialing_permissions(self):
        """
        :rtype: twilio.rest.voice.v1.dialing_permissions.DialingPermissionsList
        """
        return self.v1.dialing_permissions

    @property
    def ip_records(self):
        """
        :rtype: twilio.rest.voice.v1.ip_record.IpRecordList
        """
        return self.v1.ip_records

    @property
    def source_ip_mappings(self):
        """
        :rtype: twilio.rest.voice.v1.source_ip_mapping.SourceIpMappingList
        """
        return self.v1.source_ip_mappings

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Voice>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class UserDefinedMessageSubscriptionList(ListResource):

    def __init__(self, version, account_sid, call_sid):
        """
        Initialize the UserDefinedMessageSubscriptionList

        :param Version version: Version that contains the resource
        :param account_sid: Account SID.
        :param call_sid: Call SID.

        :returns: twilio.rest.api.v2010.account.call.user_defined_message_subscription.UserDefinedMessageSubscriptionList
        :rtype: twilio.rest.api.v2010.account.call.user_defined_message_subscription.UserDefinedMessageSubscriptionList
        """
        super(UserDefinedMessageSubscriptionList, self).__init__(version)

        # Path Solution
        self._solution = {'account_sid': account_sid, 'call_sid': call_sid, }
        self._uri = '/Accounts/{account_sid}/Calls/{call_sid}/UserDefinedMessageSubscriptions.json'.format(**self._solution)

    def create(self, callback, method, idempotency_key=values.unset):
        """
        Create the UserDefinedMessageSubscriptionInstance

        :param unicode callback: The URL we should call to send user defined messages.
        :param unicode method: HTTP method used with the callback.
        :param unicode idempotency_key: A unique string value to identify API call. This should be a unique string value per API call and can be a randomly generated.

        :returns: The created UserDefinedMessageSubscriptionInstance
        :rtype: twilio.rest.api.v2010.account.call.user_defined_message_subscription.UserDefinedMessageSubscriptionInstance
        """
        data = values.of({'Callback': callback, 'Method': method, 'IdempotencyKey': idempotency_key, })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return UserDefinedMessageSubscriptionInstance(
            self._version,
            payload,
            account_sid=self._solution['account_sid'],
            call_sid=self._solution['call_sid'],
        )

    def get(self, sid):
        """
        Constructs a UserDefinedMessageSubscriptionContext

        :param sid: User Defined Message Subscription SID.

        :returns: twilio.rest.api.v2010.account.call.user_defined_message_subscription.UserDefinedMessageSubscriptionContext
        :rtype: twilio.rest.api.v2010.account.call.user_defined_message_subscription.UserDefinedMessageSubscriptionContext
        """
        return UserDefinedMessageSubscriptionContext(
            self._version,
            account_sid=self._solution['account_sid'],
            call_sid=self._solution['call_sid'],
            sid=sid,
        )

    def __call__(self, sid):
        """
        Constructs a UserDefinedMessageSubscriptionContext

        :param sid: User Defined Message Subscription SID.

        :returns: twilio.rest.api.v2010.account.call.user_defined_message_subscription.UserDefinedMessageSubscriptionContext
        :rtype: twilio.rest.api.v2010.account.call.user_defined_message_subscription.UserDefinedMessageSubscriptionContext
        """
        return UserDefinedMessageSubscriptionContext(
            self._version,
            account_sid=self._solution['account_sid'],
            call_sid=self._solution['call_sid'],
            sid=sid,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Api.V2010.UserDefinedMessageSubscriptionList>'


class UserDefinedMessageSubscriptionPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the UserDefinedMessageSubscriptionPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param account_sid: Account SID.
        :param call_sid: Call SID.

        :returns: twilio.rest.api.v2010.account.call.user_defined_message_subscription.UserDefinedMessageSubscriptionPage
        :rtype: twilio.rest.api.v2010.account.call.user_defined_message_subscription.UserDefinedMessageSubscriptionPage
        """
        super(UserDefinedMessageSubscriptionPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of UserDefinedMessageSubscriptionInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.api.v2010.account.call.user_defined_message_subscription.UserDefinedMessageSubscriptionInstance
        :rtype: twilio.rest.api.v2010.account.call.user_defined_message_subscription.UserDefinedMessageSubscriptionInstance
        """
        return UserDefinedMessageSubscriptionInstance(
            self._version,
            payload,
            account_sid=self._solution['account_sid'],
            call_sid=self._solution['call_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Api.V2010.UserDefinedMessageSubscriptionPage>'


class UserDefinedMessageSubscriptionContext(InstanceContext):

    def __init__(self, version, account_sid, call_sid, sid):
        """
        Initialize the UserDefinedMessageSubscriptionContext

        :param Version version: Version that contains the resource
        :param account_sid: Account SID.
        :param call_sid: Call SID.
        :param sid: User Defined Message Subscription SID.

        :returns: twilio.rest.api.v2010.account.call.user_defined_message_subscription.UserDefinedMessageSubscriptionContext
        :rtype: twilio.rest.api.v2010.account.call.user_defined_message_subscription.UserDefinedMessageSubscriptionContext
        """
        super(UserDefinedMessageSubscriptionContext, self).__init__(version)

        # Path Solution
        self._solution = {'account_sid': account_sid, 'call_sid': call_sid, 'sid': sid, }
        self._uri = '/Accounts/{account_sid}/Calls/{call_sid}/UserDefinedMessageSubscriptions/{sid}.json'.format(**self._solution)

    def delete(self):
        """
        Deletes the UserDefinedMessageSubscriptionInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Api.V2010.UserDefinedMessageSubscriptionContext {}>'.format(context)


class UserDefinedMessageSubscriptionInstance(InstanceResource):

    def __init__(self, version, payload, account_sid, call_sid, sid=None):
        """
        Initialize the UserDefinedMessageSubscriptionInstance

        :returns: twilio.rest.api.v2010.account.call.user_defined_message_subscription.UserDefinedMessageSubscriptionInstance
        :rtype: twilio.rest.api.v2010.account.call.user_defined_message_subscription.UserDefinedMessageSubscriptionInstance
        """
        super(UserDefinedMessageSubscriptionInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'call_sid': payload.get('call_sid'),
            'sid': payload.get('sid'),
            'date_created': deserialize.rfc2822_datetime(payload.get('date_created')),
            'uri': payload.get('uri'),
        }

        # Context
        self._context = None
        self._solution = {
            'account_sid': account_sid,
            'call_sid': call_sid,
            'sid': sid or self._properties['sid'],
        }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: UserDefinedMessageSubscriptionContext for this UserDefinedMessageSubscriptionInstance
        :rtype: twilio.rest.api.v2010.account.call.user_defined_message_subscription.UserDefinedMessageSubscriptionContext
        """
        if self._context is None:
            self._context = UserDefinedMessageSubscriptionContext(
                self._version,
                account_sid=self._solution['account_sid'],
                call_sid=self._solution['call_sid'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: Account SID.
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def call_sid(self):
        """
        :returns: Call SID.
        :rtype: unicode
        """
        return self._properties['call_sid']

    @property
    def sid(self):
        """
        :returns: User Defined Message Subscription SID.
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def date_created(self):
        """
        :returns: The date this User Defined Message Subscription was created.
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def uri(self):
        """
        :returns: The URI of the User Defined Message Subscription Resource, relative to `https://api.twilio.com`.
        :rtype: unicode
        """
        return self._properties['uri']

    def delete(self):
        """
        Deletes the UserDefinedMessageSubscriptionInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Api.V2010.UserDefinedMessageSubscriptionInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import serialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class UserConversationList(ListResource):

    def __init__(self, version, user_sid):
        """
        Initialize the UserConversationList

        :param Version version: Version that contains the resource
        :param user_sid: The unique ID for the User.

        :returns: twilio.rest.conversations.v1.user.user_conversation.UserConversationList
        :rtype: twilio.rest.conversations.v1.user.user_conversation.UserConversationList
        """
        super(UserConversationList, self).__init__(version)

        # Path Solution
        self._solution = {'user_sid': user_sid, }
        self._uri = '/Users/{user_sid}/Conversations'.format(**self._solution)

    def stream(self, limit=None, page_size=None):
        """
        Streams UserConversationInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.conversations.v1.user.user_conversation.UserConversationInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, limit=None, page_size=None):
        """
        Lists UserConversationInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.conversations.v1.user.user_conversation.UserConversationInstance]
        """
        return list(self.stream(limit=limit, page_size=page_size, ))

    def page(self, page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of UserConversationInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of UserConversationInstance
        :rtype: twilio.rest.conversations.v1.user.user_conversation.UserConversationPage
        """
        data = values.of({'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return UserConversationPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of UserConversationInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of UserConversationInstance
        :rtype: twilio.rest.conversations.v1.user.user_conversation.UserConversationPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return UserConversationPage(self._version, response, self._solution)

    def get(self, conversation_sid):
        """
        Constructs a UserConversationContext

        :param conversation_sid: The unique SID identifier of the Conversation.

        :returns: twilio.rest.conversations.v1.user.user_conversation.UserConversationContext
        :rtype: twilio.rest.conversations.v1.user.user_conversation.UserConversationContext
        """
        return UserConversationContext(
            self._version,
            user_sid=self._solution['user_sid'],
            conversation_sid=conversation_sid,
        )

    def __call__(self, conversation_sid):
        """
        Constructs a UserConversationContext

        :param conversation_sid: The unique SID identifier of the Conversation.

        :returns: twilio.rest.conversations.v1.user.user_conversation.UserConversationContext
        :rtype: twilio.rest.conversations.v1.user.user_conversation.UserConversationContext
        """
        return UserConversationContext(
            self._version,
            user_sid=self._solution['user_sid'],
            conversation_sid=conversation_sid,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Conversations.V1.UserConversationList>'


class UserConversationPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the UserConversationPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param user_sid: The unique ID for the User.

        :returns: twilio.rest.conversations.v1.user.user_conversation.UserConversationPage
        :rtype: twilio.rest.conversations.v1.user.user_conversation.UserConversationPage
        """
        super(UserConversationPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of UserConversationInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.conversations.v1.user.user_conversation.UserConversationInstance
        :rtype: twilio.rest.conversations.v1.user.user_conversation.UserConversationInstance
        """
        return UserConversationInstance(self._version, payload, user_sid=self._solution['user_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Conversations.V1.UserConversationPage>'


class UserConversationContext(InstanceContext):

    def __init__(self, version, user_sid, conversation_sid):
        """
        Initialize the UserConversationContext

        :param Version version: Version that contains the resource
        :param user_sid: The unique SID identifier of the User.
        :param conversation_sid: The unique SID identifier of the Conversation.

        :returns: twilio.rest.conversations.v1.user.user_conversation.UserConversationContext
        :rtype: twilio.rest.conversations.v1.user.user_conversation.UserConversationContext
        """
        super(UserConversationContext, self).__init__(version)

        # Path Solution
        self._solution = {'user_sid': user_sid, 'conversation_sid': conversation_sid, }
        self._uri = '/Users/{user_sid}/Conversations/{conversation_sid}'.format(**self._solution)

    def update(self, notification_level=values.unset,
               last_read_timestamp=values.unset,
               last_read_message_index=values.unset):
        """
        Update the UserConversationInstance

        :param UserConversationInstance.NotificationLevel notification_level: The Notification Level of this User Conversation.
        :param datetime last_read_timestamp: The date of the last message read in conversation by the user.
        :param unicode last_read_message_index: The index of the last read Message.

        :returns: The updated UserConversationInstance
        :rtype: twilio.rest.conversations.v1.user.user_conversation.UserConversationInstance
        """
        data = values.of({
            'NotificationLevel': notification_level,
            'LastReadTimestamp': serialize.iso8601_datetime(last_read_timestamp),
            'LastReadMessageIndex': last_read_message_index,
        })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return UserConversationInstance(
            self._version,
            payload,
            user_sid=self._solution['user_sid'],
            conversation_sid=self._solution['conversation_sid'],
        )

    def delete(self):
        """
        Deletes the UserConversationInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def fetch(self):
        """
        Fetch the UserConversationInstance

        :returns: The fetched UserConversationInstance
        :rtype: twilio.rest.conversations.v1.user.user_conversation.UserConversationInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return UserConversationInstance(
            self._version,
            payload,
            user_sid=self._solution['user_sid'],
            conversation_sid=self._solution['conversation_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Conversations.V1.UserConversationContext {}>'.format(context)


class UserConversationInstance(InstanceResource):

    class NotificationLevel(object):
        DEFAULT = "default"
        MUTED = "muted"

    class State(object):
        INACTIVE = "inactive"
        ACTIVE = "active"
        CLOSED = "closed"

    def __init__(self, version, payload, user_sid, conversation_sid=None):
        """
        Initialize the UserConversationInstance

        :returns: twilio.rest.conversations.v1.user.user_conversation.UserConversationInstance
        :rtype: twilio.rest.conversations.v1.user.user_conversation.UserConversationInstance
        """
        super(UserConversationInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'chat_service_sid': payload.get('chat_service_sid'),
            'conversation_sid': payload.get('conversation_sid'),
            'unread_messages_count': deserialize.integer(payload.get('unread_messages_count')),
            'last_read_message_index': deserialize.integer(payload.get('last_read_message_index')),
            'participant_sid': payload.get('participant_sid'),
            'user_sid': payload.get('user_sid'),
            'friendly_name': payload.get('friendly_name'),
            'conversation_state': payload.get('conversation_state'),
            'timers': payload.get('timers'),
            'attributes': payload.get('attributes'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'created_by': payload.get('created_by'),
            'notification_level': payload.get('notification_level'),
            'unique_name': payload.get('unique_name'),
            'url': payload.get('url'),
            'links': payload.get('links'),
        }

        # Context
        self._context = None
        self._solution = {
            'user_sid': user_sid,
            'conversation_sid': conversation_sid or self._properties['conversation_sid'],
        }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: UserConversationContext for this UserConversationInstance
        :rtype: twilio.rest.conversations.v1.user.user_conversation.UserConversationContext
        """
        if self._context is None:
            self._context = UserConversationContext(
                self._version,
                user_sid=self._solution['user_sid'],
                conversation_sid=self._solution['conversation_sid'],
            )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The unique ID of the Account responsible for this conversation.
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def chat_service_sid(self):
        """
        :returns: The unique ID of the Conversation Service this conversation belongs to.
        :rtype: unicode
        """
        return self._properties['chat_service_sid']

    @property
    def conversation_sid(self):
        """
        :returns: The unique ID of the Conversation for this User Conversation.
        :rtype: unicode
        """
        return self._properties['conversation_sid']

    @property
    def unread_messages_count(self):
        """
        :returns: The number of unread Messages in the Conversation.
        :rtype: unicode
        """
        return self._properties['unread_messages_count']

    @property
    def last_read_message_index(self):
        """
        :returns: The index of the last read Message .
        :rtype: unicode
        """
        return self._properties['last_read_message_index']

    @property
    def participant_sid(self):
        """
        :returns: Participant Sid.
        :rtype: unicode
        """
        return self._properties['participant_sid']

    @property
    def user_sid(self):
        """
        :returns: The unique ID for the User.
        :rtype: unicode
        """
        return self._properties['user_sid']

    @property
    def friendly_name(self):
        """
        :returns: The human-readable name of this conversation.
        :rtype: unicode
        """
        return self._properties['friendly_name']

    @property
    def conversation_state(self):
        """
        :returns: The current state of this User Conversation
        :rtype: UserConversationInstance.State
        """
        return self._properties['conversation_state']

    @property
    def timers(self):
        """
        :returns: Timer date values for this conversation.
        :rtype: dict
        """
        return self._properties['timers']

    @property
    def attributes(self):
        """
        :returns: An optional string metadata field you can use to store any data you wish.
        :rtype: unicode
        """
        return self._properties['attributes']

    @property
    def date_created(self):
        """
        :returns: The date that this conversation was created.
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The date that this conversation was last updated.
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def created_by(self):
        """
        :returns: Creator of this conversation.
        :rtype: unicode
        """
        return self._properties['created_by']

    @property
    def notification_level(self):
        """
        :returns: The Notification Level of this User Conversation.
        :rtype: UserConversationInstance.NotificationLevel
        """
        return self._properties['notification_level']

    @property
    def unique_name(self):
        """
        :returns: An application-defined string that uniquely identifies the Conversation resource.
        :rtype: unicode
        """
        return self._properties['unique_name']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def links(self):
        """
        :returns: Absolute URLs to access the participant and conversation of this user conversation.
        :rtype: unicode
        """
        return self._properties['links']

    def update(self, notification_level=values.unset,
               last_read_timestamp=values.unset,
               last_read_message_index=values.unset):
        """
        Update the UserConversationInstance

        :param UserConversationInstance.NotificationLevel notification_level: The Notification Level of this User Conversation.
        :param datetime last_read_timestamp: The date of the last message read in conversation by the user.
        :param unicode last_read_message_index: The index of the last read Message.

        :returns: The updated UserConversationInstance
        :rtype: twilio.rest.conversations.v1.user.user_conversation.UserConversationInstance
        """
        return self._proxy.update(
            notification_level=notification_level,
            last_read_timestamp=last_read_timestamp,
            last_read_message_index=last_read_message_index,
        )

    def delete(self):
        """
        Deletes the UserConversationInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def fetch(self):
        """
        Fetch the UserConversationInstance

        :returns: The fetched UserConversationInstance
        :rtype: twilio.rest.conversations.v1.user.user_conversation.UserConversationInstance
        """
        return self._proxy.fetch()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Conversations.V1.UserConversationInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class StreamList(ListResource):

    def __init__(self, version, account_sid, call_sid):
        """
        Initialize the StreamList

        :param Version version: Version that contains the resource
        :param account_sid: The SID of the Account that created this resource
        :param call_sid: The SID of the Call the resource is associated with

        :returns: twilio.rest.api.v2010.account.call.stream.StreamList
        :rtype: twilio.rest.api.v2010.account.call.stream.StreamList
        """
        super(StreamList, self).__init__(version)

        # Path Solution
        self._solution = {'account_sid': account_sid, 'call_sid': call_sid, }
        self._uri = '/Accounts/{account_sid}/Calls/{call_sid}/Streams.json'.format(**self._solution)

    def create(self, url, name=values.unset, track=values.unset,
               status_callback=values.unset, status_callback_method=values.unset,
               parameter1_name=values.unset, parameter1_value=values.unset,
               parameter2_name=values.unset, parameter2_value=values.unset,
               parameter3_name=values.unset, parameter3_value=values.unset,
               parameter4_name=values.unset, parameter4_value=values.unset,
               parameter5_name=values.unset, parameter5_value=values.unset,
               parameter6_name=values.unset, parameter6_value=values.unset,
               parameter7_name=values.unset, parameter7_value=values.unset,
               parameter8_name=values.unset, parameter8_value=values.unset,
               parameter9_name=values.unset, parameter9_value=values.unset,
               parameter10_name=values.unset, parameter10_value=values.unset,
               parameter11_name=values.unset, parameter11_value=values.unset,
               parameter12_name=values.unset, parameter12_value=values.unset,
               parameter13_name=values.unset, parameter13_value=values.unset,
               parameter14_name=values.unset, parameter14_value=values.unset,
               parameter15_name=values.unset, parameter15_value=values.unset,
               parameter16_name=values.unset, parameter16_value=values.unset,
               parameter17_name=values.unset, parameter17_value=values.unset,
               parameter18_name=values.unset, parameter18_value=values.unset,
               parameter19_name=values.unset, parameter19_value=values.unset,
               parameter20_name=values.unset, parameter20_value=values.unset,
               parameter21_name=values.unset, parameter21_value=values.unset,
               parameter22_name=values.unset, parameter22_value=values.unset,
               parameter23_name=values.unset, parameter23_value=values.unset,
               parameter24_name=values.unset, parameter24_value=values.unset,
               parameter25_name=values.unset, parameter25_value=values.unset,
               parameter26_name=values.unset, parameter26_value=values.unset,
               parameter27_name=values.unset, parameter27_value=values.unset,
               parameter28_name=values.unset, parameter28_value=values.unset,
               parameter29_name=values.unset, parameter29_value=values.unset,
               parameter30_name=values.unset, parameter30_value=values.unset,
               parameter31_name=values.unset, parameter31_value=values.unset,
               parameter32_name=values.unset, parameter32_value=values.unset,
               parameter33_name=values.unset, parameter33_value=values.unset,
               parameter34_name=values.unset, parameter34_value=values.unset,
               parameter35_name=values.unset, parameter35_value=values.unset,
               parameter36_name=values.unset, parameter36_value=values.unset,
               parameter37_name=values.unset, parameter37_value=values.unset,
               parameter38_name=values.unset, parameter38_value=values.unset,
               parameter39_name=values.unset, parameter39_value=values.unset,
               parameter40_name=values.unset, parameter40_value=values.unset,
               parameter41_name=values.unset, parameter41_value=values.unset,
               parameter42_name=values.unset, parameter42_value=values.unset,
               parameter43_name=values.unset, parameter43_value=values.unset,
               parameter44_name=values.unset, parameter44_value=values.unset,
               parameter45_name=values.unset, parameter45_value=values.unset,
               parameter46_name=values.unset, parameter46_value=values.unset,
               parameter47_name=values.unset, parameter47_value=values.unset,
               parameter48_name=values.unset, parameter48_value=values.unset,
               parameter49_name=values.unset, parameter49_value=values.unset,
               parameter50_name=values.unset, parameter50_value=values.unset,
               parameter51_name=values.unset, parameter51_value=values.unset,
               parameter52_name=values.unset, parameter52_value=values.unset,
               parameter53_name=values.unset, parameter53_value=values.unset,
               parameter54_name=values.unset, parameter54_value=values.unset,
               parameter55_name=values.unset, parameter55_value=values.unset,
               parameter56_name=values.unset, parameter56_value=values.unset,
               parameter57_name=values.unset, parameter57_value=values.unset,
               parameter58_name=values.unset, parameter58_value=values.unset,
               parameter59_name=values.unset, parameter59_value=values.unset,
               parameter60_name=values.unset, parameter60_value=values.unset,
               parameter61_name=values.unset, parameter61_value=values.unset,
               parameter62_name=values.unset, parameter62_value=values.unset,
               parameter63_name=values.unset, parameter63_value=values.unset,
               parameter64_name=values.unset, parameter64_value=values.unset,
               parameter65_name=values.unset, parameter65_value=values.unset,
               parameter66_name=values.unset, parameter66_value=values.unset,
               parameter67_name=values.unset, parameter67_value=values.unset,
               parameter68_name=values.unset, parameter68_value=values.unset,
               parameter69_name=values.unset, parameter69_value=values.unset,
               parameter70_name=values.unset, parameter70_value=values.unset,
               parameter71_name=values.unset, parameter71_value=values.unset,
               parameter72_name=values.unset, parameter72_value=values.unset,
               parameter73_name=values.unset, parameter73_value=values.unset,
               parameter74_name=values.unset, parameter74_value=values.unset,
               parameter75_name=values.unset, parameter75_value=values.unset,
               parameter76_name=values.unset, parameter76_value=values.unset,
               parameter77_name=values.unset, parameter77_value=values.unset,
               parameter78_name=values.unset, parameter78_value=values.unset,
               parameter79_name=values.unset, parameter79_value=values.unset,
               parameter80_name=values.unset, parameter80_value=values.unset,
               parameter81_name=values.unset, parameter81_value=values.unset,
               parameter82_name=values.unset, parameter82_value=values.unset,
               parameter83_name=values.unset, parameter83_value=values.unset,
               parameter84_name=values.unset, parameter84_value=values.unset,
               parameter85_name=values.unset, parameter85_value=values.unset,
               parameter86_name=values.unset, parameter86_value=values.unset,
               parameter87_name=values.unset, parameter87_value=values.unset,
               parameter88_name=values.unset, parameter88_value=values.unset,
               parameter89_name=values.unset, parameter89_value=values.unset,
               parameter90_name=values.unset, parameter90_value=values.unset,
               parameter91_name=values.unset, parameter91_value=values.unset,
               parameter92_name=values.unset, parameter92_value=values.unset,
               parameter93_name=values.unset, parameter93_value=values.unset,
               parameter94_name=values.unset, parameter94_value=values.unset,
               parameter95_name=values.unset, parameter95_value=values.unset,
               parameter96_name=values.unset, parameter96_value=values.unset,
               parameter97_name=values.unset, parameter97_value=values.unset,
               parameter98_name=values.unset, parameter98_value=values.unset,
               parameter99_name=values.unset, parameter99_value=values.unset):
        """
        Create the StreamInstance

        :param unicode url: Url where WebSocket connection will be established.
        :param unicode name: The name of this resource
        :param StreamInstance.Track track: One of `inbound_track`, `outbound_track`, `both_tracks`.
        :param unicode status_callback: Absolute URL of the status callback.
        :param unicode status_callback_method: The http method for the status_callback.
        :param unicode parameter1_name: Parameter name
        :param unicode parameter1_value: Parameter value
        :param unicode parameter2_name: Parameter name
        :param unicode parameter2_value: Parameter value
        :param unicode parameter3_name: Parameter name
        :param unicode parameter3_value: Parameter value
        :param unicode parameter4_name: Parameter name
        :param unicode parameter4_value: Parameter value
        :param unicode parameter5_name: Parameter name
        :param unicode parameter5_value: Parameter value
        :param unicode parameter6_name: Parameter name
        :param unicode parameter6_value: Parameter value
        :param unicode parameter7_name: Parameter name
        :param unicode parameter7_value: Parameter value
        :param unicode parameter8_name: Parameter name
        :param unicode parameter8_value: Parameter value
        :param unicode parameter9_name: Parameter name
        :param unicode parameter9_value: Parameter value
        :param unicode parameter10_name: Parameter name
        :param unicode parameter10_value: Parameter value
        :param unicode parameter11_name: Parameter name
        :param unicode parameter11_value: Parameter value
        :param unicode parameter12_name: Parameter name
        :param unicode parameter12_value: Parameter value
        :param unicode parameter13_name: Parameter name
        :param unicode parameter13_value: Parameter value
        :param unicode parameter14_name: Parameter name
        :param unicode parameter14_value: Parameter value
        :param unicode parameter15_name: Parameter name
        :param unicode parameter15_value: Parameter value
        :param unicode parameter16_name: Parameter name
        :param unicode parameter16_value: Parameter value
        :param unicode parameter17_name: Parameter name
        :param unicode parameter17_value: Parameter value
        :param unicode parameter18_name: Parameter name
        :param unicode parameter18_value: Parameter value
        :param unicode parameter19_name: Parameter name
        :param unicode parameter19_value: Parameter value
        :param unicode parameter20_name: Parameter name
        :param unicode parameter20_value: Parameter value
        :param unicode parameter21_name: Parameter name
        :param unicode parameter21_value: Parameter value
        :param unicode parameter22_name: Parameter name
        :param unicode parameter22_value: Parameter value
        :param unicode parameter23_name: Parameter name
        :param unicode parameter23_value: Parameter value
        :param unicode parameter24_name: Parameter name
        :param unicode parameter24_value: Parameter value
        :param unicode parameter25_name: Parameter name
        :param unicode parameter25_value: Parameter value
        :param unicode parameter26_name: Parameter name
        :param unicode parameter26_value: Parameter value
        :param unicode parameter27_name: Parameter name
        :param unicode parameter27_value: Parameter value
        :param unicode parameter28_name: Parameter name
        :param unicode parameter28_value: Parameter value
        :param unicode parameter29_name: Parameter name
        :param unicode parameter29_value: Parameter value
        :param unicode parameter30_name: Parameter name
        :param unicode parameter30_value: Parameter value
        :param unicode parameter31_name: Parameter name
        :param unicode parameter31_value: Parameter value
        :param unicode parameter32_name: Parameter name
        :param unicode parameter32_value: Parameter value
        :param unicode parameter33_name: Parameter name
        :param unicode parameter33_value: Parameter value
        :param unicode parameter34_name: Parameter name
        :param unicode parameter34_value: Parameter value
        :param unicode parameter35_name: Parameter name
        :param unicode parameter35_value: Parameter value
        :param unicode parameter36_name: Parameter name
        :param unicode parameter36_value: Parameter value
        :param unicode parameter37_name: Parameter name
        :param unicode parameter37_value: Parameter value
        :param unicode parameter38_name: Parameter name
        :param unicode parameter38_value: Parameter value
        :param unicode parameter39_name: Parameter name
        :param unicode parameter39_value: Parameter value
        :param unicode parameter40_name: Parameter name
        :param unicode parameter40_value: Parameter value
        :param unicode parameter41_name: Parameter name
        :param unicode parameter41_value: Parameter value
        :param unicode parameter42_name: Parameter name
        :param unicode parameter42_value: Parameter value
        :param unicode parameter43_name: Parameter name
        :param unicode parameter43_value: Parameter value
        :param unicode parameter44_name: Parameter name
        :param unicode parameter44_value: Parameter value
        :param unicode parameter45_name: Parameter name
        :param unicode parameter45_value: Parameter value
        :param unicode parameter46_name: Parameter name
        :param unicode parameter46_value: Parameter value
        :param unicode parameter47_name: Parameter name
        :param unicode parameter47_value: Parameter value
        :param unicode parameter48_name: Parameter name
        :param unicode parameter48_value: Parameter value
        :param unicode parameter49_name: Parameter name
        :param unicode parameter49_value: Parameter value
        :param unicode parameter50_name: Parameter name
        :param unicode parameter50_value: Parameter value
        :param unicode parameter51_name: Parameter name
        :param unicode parameter51_value: Parameter value
        :param unicode parameter52_name: Parameter name
        :param unicode parameter52_value: Parameter value
        :param unicode parameter53_name: Parameter name
        :param unicode parameter53_value: Parameter value
        :param unicode parameter54_name: Parameter name
        :param unicode parameter54_value: Parameter value
        :param unicode parameter55_name: Parameter name
        :param unicode parameter55_value: Parameter value
        :param unicode parameter56_name: Parameter name
        :param unicode parameter56_value: Parameter value
        :param unicode parameter57_name: Parameter name
        :param unicode parameter57_value: Parameter value
        :param unicode parameter58_name: Parameter name
        :param unicode parameter58_value: Parameter value
        :param unicode parameter59_name: Parameter name
        :param unicode parameter59_value: Parameter value
        :param unicode parameter60_name: Parameter name
        :param unicode parameter60_value: Parameter value
        :param unicode parameter61_name: Parameter name
        :param unicode parameter61_value: Parameter value
        :param unicode parameter62_name: Parameter name
        :param unicode parameter62_value: Parameter value
        :param unicode parameter63_name: Parameter name
        :param unicode parameter63_value: Parameter value
        :param unicode parameter64_name: Parameter name
        :param unicode parameter64_value: Parameter value
        :param unicode parameter65_name: Parameter name
        :param unicode parameter65_value: Parameter value
        :param unicode parameter66_name: Parameter name
        :param unicode parameter66_value: Parameter value
        :param unicode parameter67_name: Parameter name
        :param unicode parameter67_value: Parameter value
        :param unicode parameter68_name: Parameter name
        :param unicode parameter68_value: Parameter value
        :param unicode parameter69_name: Parameter name
        :param unicode parameter69_value: Parameter value
        :param unicode parameter70_name: Parameter name
        :param unicode parameter70_value: Parameter value
        :param unicode parameter71_name: Parameter name
        :param unicode parameter71_value: Parameter value
        :param unicode parameter72_name: Parameter name
        :param unicode parameter72_value: Parameter value
        :param unicode parameter73_name: Parameter name
        :param unicode parameter73_value: Parameter value
        :param unicode parameter74_name: Parameter name
        :param unicode parameter74_value: Parameter value
        :param unicode parameter75_name: Parameter name
        :param unicode parameter75_value: Parameter value
        :param unicode parameter76_name: Parameter name
        :param unicode parameter76_value: Parameter value
        :param unicode parameter77_name: Parameter name
        :param unicode parameter77_value: Parameter value
        :param unicode parameter78_name: Parameter name
        :param unicode parameter78_value: Parameter value
        :param unicode parameter79_name: Parameter name
        :param unicode parameter79_value: Parameter value
        :param unicode parameter80_name: Parameter name
        :param unicode parameter80_value: Parameter value
        :param unicode parameter81_name: Parameter name
        :param unicode parameter81_value: Parameter value
        :param unicode parameter82_name: Parameter name
        :param unicode parameter82_value: Parameter value
        :param unicode parameter83_name: Parameter name
        :param unicode parameter83_value: Parameter value
        :param unicode parameter84_name: Parameter name
        :param unicode parameter84_value: Parameter value
        :param unicode parameter85_name: Parameter name
        :param unicode parameter85_value: Parameter value
        :param unicode parameter86_name: Parameter name
        :param unicode parameter86_value: Parameter value
        :param unicode parameter87_name: Parameter name
        :param unicode parameter87_value: Parameter value
        :param unicode parameter88_name: Parameter name
        :param unicode parameter88_value: Parameter value
        :param unicode parameter89_name: Parameter name
        :param unicode parameter89_value: Parameter value
        :param unicode parameter90_name: Parameter name
        :param unicode parameter90_value: Parameter value
        :param unicode parameter91_name: Parameter name
        :param unicode parameter91_value: Parameter value
        :param unicode parameter92_name: Parameter name
        :param unicode parameter92_value: Parameter value
        :param unicode parameter93_name: Parameter name
        :param unicode parameter93_value: Parameter value
        :param unicode parameter94_name: Parameter name
        :param unicode parameter94_value: Parameter value
        :param unicode parameter95_name: Parameter name
        :param unicode parameter95_value: Parameter value
        :param unicode parameter96_name: Parameter name
        :param unicode parameter96_value: Parameter value
        :param unicode parameter97_name: Parameter name
        :param unicode parameter97_value: Parameter value
        :param unicode parameter98_name: Parameter name
        :param unicode parameter98_value: Parameter value
        :param unicode parameter99_name: Parameter name
        :param unicode parameter99_value: Parameter value

        :returns: The created StreamInstance
        :rtype: twilio.rest.api.v2010.account.call.stream.StreamInstance
        """
        data = values.of({
            'Url': url,
            'Name': name,
            'Track': track,
            'StatusCallback': status_callback,
            'StatusCallbackMethod': status_callback_method,
            'Parameter1.Name': parameter1_name,
            'Parameter1.Value': parameter1_value,
            'Parameter2.Name': parameter2_name,
            'Parameter2.Value': parameter2_value,
            'Parameter3.Name': parameter3_name,
            'Parameter3.Value': parameter3_value,
            'Parameter4.Name': parameter4_name,
            'Parameter4.Value': parameter4_value,
            'Parameter5.Name': parameter5_name,
            'Parameter5.Value': parameter5_value,
            'Parameter6.Name': parameter6_name,
            'Parameter6.Value': parameter6_value,
            'Parameter7.Name': parameter7_name,
            'Parameter7.Value': parameter7_value,
            'Parameter8.Name': parameter8_name,
            'Parameter8.Value': parameter8_value,
            'Parameter9.Name': parameter9_name,
            'Parameter9.Value': parameter9_value,
            'Parameter10.Name': parameter10_name,
            'Parameter10.Value': parameter10_value,
            'Parameter11.Name': parameter11_name,
            'Parameter11.Value': parameter11_value,
            'Parameter12.Name': parameter12_name,
            'Parameter12.Value': parameter12_value,
            'Parameter13.Name': parameter13_name,
            'Parameter13.Value': parameter13_value,
            'Parameter14.Name': parameter14_name,
            'Parameter14.Value': parameter14_value,
            'Parameter15.Name': parameter15_name,
            'Parameter15.Value': parameter15_value,
            'Parameter16.Name': parameter16_name,
            'Parameter16.Value': parameter16_value,
            'Parameter17.Name': parameter17_name,
            'Parameter17.Value': parameter17_value,
            'Parameter18.Name': parameter18_name,
            'Parameter18.Value': parameter18_value,
            'Parameter19.Name': parameter19_name,
            'Parameter19.Value': parameter19_value,
            'Parameter20.Name': parameter20_name,
            'Parameter20.Value': parameter20_value,
            'Parameter21.Name': parameter21_name,
            'Parameter21.Value': parameter21_value,
            'Parameter22.Name': parameter22_name,
            'Parameter22.Value': parameter22_value,
            'Parameter23.Name': parameter23_name,
            'Parameter23.Value': parameter23_value,
            'Parameter24.Name': parameter24_name,
            'Parameter24.Value': parameter24_value,
            'Parameter25.Name': parameter25_name,
            'Parameter25.Value': parameter25_value,
            'Parameter26.Name': parameter26_name,
            'Parameter26.Value': parameter26_value,
            'Parameter27.Name': parameter27_name,
            'Parameter27.Value': parameter27_value,
            'Parameter28.Name': parameter28_name,
            'Parameter28.Value': parameter28_value,
            'Parameter29.Name': parameter29_name,
            'Parameter29.Value': parameter29_value,
            'Parameter30.Name': parameter30_name,
            'Parameter30.Value': parameter30_value,
            'Parameter31.Name': parameter31_name,
            'Parameter31.Value': parameter31_value,
            'Parameter32.Name': parameter32_name,
            'Parameter32.Value': parameter32_value,
            'Parameter33.Name': parameter33_name,
            'Parameter33.Value': parameter33_value,
            'Parameter34.Name': parameter34_name,
            'Parameter34.Value': parameter34_value,
            'Parameter35.Name': parameter35_name,
            'Parameter35.Value': parameter35_value,
            'Parameter36.Name': parameter36_name,
            'Parameter36.Value': parameter36_value,
            'Parameter37.Name': parameter37_name,
            'Parameter37.Value': parameter37_value,
            'Parameter38.Name': parameter38_name,
            'Parameter38.Value': parameter38_value,
            'Parameter39.Name': parameter39_name,
            'Parameter39.Value': parameter39_value,
            'Parameter40.Name': parameter40_name,
            'Parameter40.Value': parameter40_value,
            'Parameter41.Name': parameter41_name,
            'Parameter41.Value': parameter41_value,
            'Parameter42.Name': parameter42_name,
            'Parameter42.Value': parameter42_value,
            'Parameter43.Name': parameter43_name,
            'Parameter43.Value': parameter43_value,
            'Parameter44.Name': parameter44_name,
            'Parameter44.Value': parameter44_value,
            'Parameter45.Name': parameter45_name,
            'Parameter45.Value': parameter45_value,
            'Parameter46.Name': parameter46_name,
            'Parameter46.Value': parameter46_value,
            'Parameter47.Name': parameter47_name,
            'Parameter47.Value': parameter47_value,
            'Parameter48.Name': parameter48_name,
            'Parameter48.Value': parameter48_value,
            'Parameter49.Name': parameter49_name,
            'Parameter49.Value': parameter49_value,
            'Parameter50.Name': parameter50_name,
            'Parameter50.Value': parameter50_value,
            'Parameter51.Name': parameter51_name,
            'Parameter51.Value': parameter51_value,
            'Parameter52.Name': parameter52_name,
            'Parameter52.Value': parameter52_value,
            'Parameter53.Name': parameter53_name,
            'Parameter53.Value': parameter53_value,
            'Parameter54.Name': parameter54_name,
            'Parameter54.Value': parameter54_value,
            'Parameter55.Name': parameter55_name,
            'Parameter55.Value': parameter55_value,
            'Parameter56.Name': parameter56_name,
            'Parameter56.Value': parameter56_value,
            'Parameter57.Name': parameter57_name,
            'Parameter57.Value': parameter57_value,
            'Parameter58.Name': parameter58_name,
            'Parameter58.Value': parameter58_value,
            'Parameter59.Name': parameter59_name,
            'Parameter59.Value': parameter59_value,
            'Parameter60.Name': parameter60_name,
            'Parameter60.Value': parameter60_value,
            'Parameter61.Name': parameter61_name,
            'Parameter61.Value': parameter61_value,
            'Parameter62.Name': parameter62_name,
            'Parameter62.Value': parameter62_value,
            'Parameter63.Name': parameter63_name,
            'Parameter63.Value': parameter63_value,
            'Parameter64.Name': parameter64_name,
            'Parameter64.Value': parameter64_value,
            'Parameter65.Name': parameter65_name,
            'Parameter65.Value': parameter65_value,
            'Parameter66.Name': parameter66_name,
            'Parameter66.Value': parameter66_value,
            'Parameter67.Name': parameter67_name,
            'Parameter67.Value': parameter67_value,
            'Parameter68.Name': parameter68_name,
            'Parameter68.Value': parameter68_value,
            'Parameter69.Name': parameter69_name,
            'Parameter69.Value': parameter69_value,
            'Parameter70.Name': parameter70_name,
            'Parameter70.Value': parameter70_value,
            'Parameter71.Name': parameter71_name,
            'Parameter71.Value': parameter71_value,
            'Parameter72.Name': parameter72_name,
            'Parameter72.Value': parameter72_value,
            'Parameter73.Name': parameter73_name,
            'Parameter73.Value': parameter73_value,
            'Parameter74.Name': parameter74_name,
            'Parameter74.Value': parameter74_value,
            'Parameter75.Name': parameter75_name,
            'Parameter75.Value': parameter75_value,
            'Parameter76.Name': parameter76_name,
            'Parameter76.Value': parameter76_value,
            'Parameter77.Name': parameter77_name,
            'Parameter77.Value': parameter77_value,
            'Parameter78.Name': parameter78_name,
            'Parameter78.Value': parameter78_value,
            'Parameter79.Name': parameter79_name,
            'Parameter79.Value': parameter79_value,
            'Parameter80.Name': parameter80_name,
            'Parameter80.Value': parameter80_value,
            'Parameter81.Name': parameter81_name,
            'Parameter81.Value': parameter81_value,
            'Parameter82.Name': parameter82_name,
            'Parameter82.Value': parameter82_value,
            'Parameter83.Name': parameter83_name,
            'Parameter83.Value': parameter83_value,
            'Parameter84.Name': parameter84_name,
            'Parameter84.Value': parameter84_value,
            'Parameter85.Name': parameter85_name,
            'Parameter85.Value': parameter85_value,
            'Parameter86.Name': parameter86_name,
            'Parameter86.Value': parameter86_value,
            'Parameter87.Name': parameter87_name,
            'Parameter87.Value': parameter87_value,
            'Parameter88.Name': parameter88_name,
            'Parameter88.Value': parameter88_value,
            'Parameter89.Name': parameter89_name,
            'Parameter89.Value': parameter89_value,
            'Parameter90.Name': parameter90_name,
            'Parameter90.Value': parameter90_value,
            'Parameter91.Name': parameter91_name,
            'Parameter91.Value': parameter91_value,
            'Parameter92.Name': parameter92_name,
            'Parameter92.Value': parameter92_value,
            'Parameter93.Name': parameter93_name,
            'Parameter93.Value': parameter93_value,
            'Parameter94.Name': parameter94_name,
            'Parameter94.Value': parameter94_value,
            'Parameter95.Name': parameter95_name,
            'Parameter95.Value': parameter95_value,
            'Parameter96.Name': parameter96_name,
            'Parameter96.Value': parameter96_value,
            'Parameter97.Name': parameter97_name,
            'Parameter97.Value': parameter97_value,
            'Parameter98.Name': parameter98_name,
            'Parameter98.Value': parameter98_value,
            'Parameter99.Name': parameter99_name,
            'Parameter99.Value': parameter99_value,
        })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return StreamInstance(
            self._version,
            payload,
            account_sid=self._solution['account_sid'],
            call_sid=self._solution['call_sid'],
        )

    def get(self, sid):
        """
        Constructs a StreamContext

        :param sid: The SID of the Stream resource, or the `name`

        :returns: twilio.rest.api.v2010.account.call.stream.StreamContext
        :rtype: twilio.rest.api.v2010.account.call.stream.StreamContext
        """
        return StreamContext(
            self._version,
            account_sid=self._solution['account_sid'],
            call_sid=self._solution['call_sid'],
            sid=sid,
        )

    def __call__(self, sid):
        """
        Constructs a StreamContext

        :param sid: The SID of the Stream resource, or the `name`

        :returns: twilio.rest.api.v2010.account.call.stream.StreamContext
        :rtype: twilio.rest.api.v2010.account.call.stream.StreamContext
        """
        return StreamContext(
            self._version,
            account_sid=self._solution['account_sid'],
            call_sid=self._solution['call_sid'],
            sid=sid,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Api.V2010.StreamList>'


class StreamPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the StreamPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param account_sid: The SID of the Account that created this resource
        :param call_sid: The SID of the Call the resource is associated with

        :returns: twilio.rest.api.v2010.account.call.stream.StreamPage
        :rtype: twilio.rest.api.v2010.account.call.stream.StreamPage
        """
        super(StreamPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of StreamInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.api.v2010.account.call.stream.StreamInstance
        :rtype: twilio.rest.api.v2010.account.call.stream.StreamInstance
        """
        return StreamInstance(
            self._version,
            payload,
            account_sid=self._solution['account_sid'],
            call_sid=self._solution['call_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Api.V2010.StreamPage>'


class StreamContext(InstanceContext):

    def __init__(self, version, account_sid, call_sid, sid):
        """
        Initialize the StreamContext

        :param Version version: Version that contains the resource
        :param account_sid: The SID of the Account that created this resource
        :param call_sid: The SID of the Call the resource is associated with
        :param sid: The SID of the Stream resource, or the `name`

        :returns: twilio.rest.api.v2010.account.call.stream.StreamContext
        :rtype: twilio.rest.api.v2010.account.call.stream.StreamContext
        """
        super(StreamContext, self).__init__(version)

        # Path Solution
        self._solution = {'account_sid': account_sid, 'call_sid': call_sid, 'sid': sid, }
        self._uri = '/Accounts/{account_sid}/Calls/{call_sid}/Streams/{sid}.json'.format(**self._solution)

    def update(self, status):
        """
        Update the StreamInstance

        :param StreamInstance.UpdateStatus status: The status. Must have the value `stopped`

        :returns: The updated StreamInstance
        :rtype: twilio.rest.api.v2010.account.call.stream.StreamInstance
        """
        data = values.of({'Status': status, })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return StreamInstance(
            self._version,
            payload,
            account_sid=self._solution['account_sid'],
            call_sid=self._solution['call_sid'],
            sid=self._solution['sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Api.V2010.StreamContext {}>'.format(context)


class StreamInstance(InstanceResource):

    class Track(object):
        INBOUND_TRACK = "inbound_track"
        OUTBOUND_TRACK = "outbound_track"
        BOTH_TRACKS = "both_tracks"

    class Status(object):
        IN_PROGRESS = "in-progress"
        STOPPED = "stopped"

    class UpdateStatus(object):
        STOPPED = "stopped"

    def __init__(self, version, payload, account_sid, call_sid, sid=None):
        """
        Initialize the StreamInstance

        :returns: twilio.rest.api.v2010.account.call.stream.StreamInstance
        :rtype: twilio.rest.api.v2010.account.call.stream.StreamInstance
        """
        super(StreamInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'account_sid': payload.get('account_sid'),
            'call_sid': payload.get('call_sid'),
            'name': payload.get('name'),
            'status': payload.get('status'),
            'date_updated': deserialize.rfc2822_datetime(payload.get('date_updated')),
            'uri': payload.get('uri'),
        }

        # Context
        self._context = None
        self._solution = {
            'account_sid': account_sid,
            'call_sid': call_sid,
            'sid': sid or self._properties['sid'],
        }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: StreamContext for this StreamInstance
        :rtype: twilio.rest.api.v2010.account.call.stream.StreamContext
        """
        if self._context is None:
            self._context = StreamContext(
                self._version,
                account_sid=self._solution['account_sid'],
                call_sid=self._solution['call_sid'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def sid(self):
        """
        :returns: The SID of the Stream resource.
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created this resource
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def call_sid(self):
        """
        :returns: The SID of the Call the resource is associated with
        :rtype: unicode
        """
        return self._properties['call_sid']

    @property
    def name(self):
        """
        :returns: The name of this resource
        :rtype: unicode
        """
        return self._properties['name']

    @property
    def status(self):
        """
        :returns: The status - one of `stopped`, `in-progress`
        :rtype: StreamInstance.Status
        """
        return self._properties['status']

    @property
    def date_updated(self):
        """
        :returns: The RFC 2822 date and time in GMT that this resource was last updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def uri(self):
        """
        :returns: The URI of the resource, relative to `https://api.twilio.com`
        :rtype: unicode
        """
        return self._properties['uri']

    def update(self, status):
        """
        Update the StreamInstance

        :param StreamInstance.UpdateStatus status: The status. Must have the value `stopped`

        :returns: The updated StreamInstance
        :rtype: twilio.rest.api.v2010.account.call.stream.StreamInstance
        """
        return self._proxy.update(status, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Api.V2010.StreamInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page
from twilio.rest.voice.v1.dialing_permissions.bulk_country_update import BulkCountryUpdateList
from twilio.rest.voice.v1.dialing_permissions.country import CountryList
from twilio.rest.voice.v1.dialing_permissions.settings import SettingsList


class DialingPermissionsList(ListResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version):
        """
        Initialize the DialingPermissionsList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.voice.v1.dialing_permissions.DialingPermissionsList
        :rtype: twilio.rest.voice.v1.dialing_permissions.DialingPermissionsList
        """
        super(DialingPermissionsList, self).__init__(version)

        # Path Solution
        self._solution = {}

        # Components
        self._countries = None
        self._settings = None
        self._bulk_country_updates = None

    @property
    def countries(self):
        """
        Access the countries

        :returns: twilio.rest.voice.v1.dialing_permissions.country.CountryList
        :rtype: twilio.rest.voice.v1.dialing_permissions.country.CountryList
        """
        if self._countries is None:
            self._countries = CountryList(self._version, )
        return self._countries

    @property
    def settings(self):
        """
        Access the settings

        :returns: twilio.rest.voice.v1.dialing_permissions.settings.SettingsList
        :rtype: twilio.rest.voice.v1.dialing_permissions.settings.SettingsList
        """
        if self._settings is None:
            self._settings = SettingsList(self._version, )
        return self._settings

    @property
    def bulk_country_updates(self):
        """
        Access the bulk_country_updates

        :returns: twilio.rest.voice.v1.dialing_permissions.bulk_country_update.BulkCountryUpdateList
        :rtype: twilio.rest.voice.v1.dialing_permissions.bulk_country_update.BulkCountryUpdateList
        """
        if self._bulk_country_updates is None:
            self._bulk_country_updates = BulkCountryUpdateList(self._version, )
        return self._bulk_country_updates

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Voice.V1.DialingPermissionsList>'


class DialingPermissionsPage(Page):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, response, solution):
        """
        Initialize the DialingPermissionsPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.voice.v1.dialing_permissions.DialingPermissionsPage
        :rtype: twilio.rest.voice.v1.dialing_permissions.DialingPermissionsPage
        """
        super(DialingPermissionsPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of DialingPermissionsInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.voice.v1.dialing_permissions.DialingPermissionsInstance
        :rtype: twilio.rest.voice.v1.dialing_permissions.DialingPermissionsInstance
        """
        return DialingPermissionsInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Voice.V1.DialingPermissionsPage>'


class DialingPermissionsInstance(InstanceResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, payload):
        """
        Initialize the DialingPermissionsInstance

        :returns: twilio.rest.voice.v1.dialing_permissions.DialingPermissionsInstance
        :rtype: twilio.rest.voice.v1.dialing_permissions.DialingPermissionsInstance
        """
        super(DialingPermissionsInstance, self).__init__(version)

        # Context
        self._context = None
        self._solution = {}

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Voice.V1.DialingPermissionsInstance>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class AnnotationList(ListResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, call_sid):
        """
        Initialize the AnnotationList

        :param Version version: Version that contains the resource
        :param call_sid: Call SID.

        :returns: twilio.rest.insights.v1.call.annotation.AnnotationList
        :rtype: twilio.rest.insights.v1.call.annotation.AnnotationList
        """
        super(AnnotationList, self).__init__(version)

        # Path Solution
        self._solution = {'call_sid': call_sid, }

    def get(self):
        """
        Constructs a AnnotationContext

        :returns: twilio.rest.insights.v1.call.annotation.AnnotationContext
        :rtype: twilio.rest.insights.v1.call.annotation.AnnotationContext
        """
        return AnnotationContext(self._version, call_sid=self._solution['call_sid'], )

    def __call__(self):
        """
        Constructs a AnnotationContext

        :returns: twilio.rest.insights.v1.call.annotation.AnnotationContext
        :rtype: twilio.rest.insights.v1.call.annotation.AnnotationContext
        """
        return AnnotationContext(self._version, call_sid=self._solution['call_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Insights.V1.AnnotationList>'


class AnnotationPage(Page):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, response, solution):
        """
        Initialize the AnnotationPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param call_sid: Call SID.

        :returns: twilio.rest.insights.v1.call.annotation.AnnotationPage
        :rtype: twilio.rest.insights.v1.call.annotation.AnnotationPage
        """
        super(AnnotationPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of AnnotationInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.insights.v1.call.annotation.AnnotationInstance
        :rtype: twilio.rest.insights.v1.call.annotation.AnnotationInstance
        """
        return AnnotationInstance(self._version, payload, call_sid=self._solution['call_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Insights.V1.AnnotationPage>'


class AnnotationContext(InstanceContext):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, call_sid):
        """
        Initialize the AnnotationContext

        :param Version version: Version that contains the resource
        :param call_sid: Call SID.

        :returns: twilio.rest.insights.v1.call.annotation.AnnotationContext
        :rtype: twilio.rest.insights.v1.call.annotation.AnnotationContext
        """
        super(AnnotationContext, self).__init__(version)

        # Path Solution
        self._solution = {'call_sid': call_sid, }
        self._uri = '/Voice/{call_sid}/Annotation'.format(**self._solution)

    def update(self, answered_by=values.unset, connectivity_issue=values.unset,
               quality_issues=values.unset, spam=values.unset,
               call_score=values.unset, comment=values.unset,
               incident=values.unset):
        """
        Update the AnnotationInstance

        :param AnnotationInstance.AnsweredBy answered_by: Indicates the answering entity as determined by Answering Machine Detection.
        :param AnnotationInstance.ConnectivityIssue connectivity_issue: Indicates if the call had any connectivity issue
        :param unicode quality_issues: Indicates if the call had audio quality issues.
        :param bool spam: Call spam indicator
        :param unicode call_score: Call Score
        :param unicode comment: User comments
        :param unicode incident: Call tag for incidents or support ticket

        :returns: The updated AnnotationInstance
        :rtype: twilio.rest.insights.v1.call.annotation.AnnotationInstance
        """
        data = values.of({
            'AnsweredBy': answered_by,
            'ConnectivityIssue': connectivity_issue,
            'QualityIssues': quality_issues,
            'Spam': spam,
            'CallScore': call_score,
            'Comment': comment,
            'Incident': incident,
        })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return AnnotationInstance(self._version, payload, call_sid=self._solution['call_sid'], )

    def fetch(self):
        """
        Fetch the AnnotationInstance

        :returns: The fetched AnnotationInstance
        :rtype: twilio.rest.insights.v1.call.annotation.AnnotationInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return AnnotationInstance(self._version, payload, call_sid=self._solution['call_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Insights.V1.AnnotationContext {}>'.format(context)


class AnnotationInstance(InstanceResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    class AnsweredBy(object):
        UNKNOWN_ANSWERED_BY = "unknown_answered_by"
        HUMAN = "human"
        MACHINE = "machine"

    class ConnectivityIssue(object):
        UNKNOWN_CONNECTIVITY_ISSUE = "unknown_connectivity_issue"
        NO_CONNECTIVITY_ISSUE = "no_connectivity_issue"
        INVALID_NUMBER = "invalid_number"
        CALLER_ID = "caller_id"
        DROPPED_CALL = "dropped_call"
        NUMBER_REACHABILITY = "number_reachability"

    class QualityIssues(object):
        UNKNOWN_QUALITY_ISSUE = "unknown_quality_issue"
        NO_QUALITY_ISSUE = "no_quality_issue"
        LOW_VOLUME = "low_volume"
        CHOPPY_ROBOTIC = "choppy_robotic"
        ECHO = "echo"
        DTMF = "dtmf"
        LATENCY = "latency"
        OWA = "owa"
        STATIC_NOISE = "static_noise"

    def __init__(self, version, payload, call_sid):
        """
        Initialize the AnnotationInstance

        :returns: twilio.rest.insights.v1.call.annotation.AnnotationInstance
        :rtype: twilio.rest.insights.v1.call.annotation.AnnotationInstance
        """
        super(AnnotationInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'call_sid': payload.get('call_sid'),
            'account_sid': payload.get('account_sid'),
            'answered_by': payload.get('answered_by'),
            'connectivity_issue': payload.get('connectivity_issue'),
            'quality_issues': payload.get('quality_issues'),
            'spam': payload.get('spam'),
            'call_score': deserialize.integer(payload.get('call_score')),
            'comment': payload.get('comment'),
            'incident': payload.get('incident'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {'call_sid': call_sid, }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: AnnotationContext for this AnnotationInstance
        :rtype: twilio.rest.insights.v1.call.annotation.AnnotationContext
        """
        if self._context is None:
            self._context = AnnotationContext(self._version, call_sid=self._solution['call_sid'], )
        return self._context

    @property
    def call_sid(self):
        """
        :returns: Call SID.
        :rtype: unicode
        """
        return self._properties['call_sid']

    @property
    def account_sid(self):
        """
        :returns: Account SID.
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def answered_by(self):
        """
        :returns: Indicates the answering entity as determined by Answering Machine Detection.
        :rtype: AnnotationInstance.AnsweredBy
        """
        return self._properties['answered_by']

    @property
    def connectivity_issue(self):
        """
        :returns: Indicates if the call had any connectivity issue
        :rtype: AnnotationInstance.ConnectivityIssue
        """
        return self._properties['connectivity_issue']

    @property
    def quality_issues(self):
        """
        :returns: Indicates if the call had audio quality issues.
        :rtype: list[unicode]
        """
        return self._properties['quality_issues']

    @property
    def spam(self):
        """
        :returns: Call spam indicator
        :rtype: bool
        """
        return self._properties['spam']

    @property
    def call_score(self):
        """
        :returns: Call Score
        :rtype: unicode
        """
        return self._properties['call_score']

    @property
    def comment(self):
        """
        :returns: User comments
        :rtype: unicode
        """
        return self._properties['comment']

    @property
    def incident(self):
        """
        :returns: Call tag for incidents or support ticket
        :rtype: unicode
        """
        return self._properties['incident']

    @property
    def url(self):
        """
        :returns: The URL of this resource.
        :rtype: unicode
        """
        return self._properties['url']

    def update(self, answered_by=values.unset, connectivity_issue=values.unset,
               quality_issues=values.unset, spam=values.unset,
               call_score=values.unset, comment=values.unset,
               incident=values.unset):
        """
        Update the AnnotationInstance

        :param AnnotationInstance.AnsweredBy answered_by: Indicates the answering entity as determined by Answering Machine Detection.
        :param AnnotationInstance.ConnectivityIssue connectivity_issue: Indicates if the call had any connectivity issue
        :param unicode quality_issues: Indicates if the call had audio quality issues.
        :param bool spam: Call spam indicator
        :param unicode call_score: Call Score
        :param unicode comment: User comments
        :param unicode incident: Call tag for incidents or support ticket

        :returns: The updated AnnotationInstance
        :rtype: twilio.rest.insights.v1.call.annotation.AnnotationInstance
        """
        return self._proxy.update(
            answered_by=answered_by,
            connectivity_issue=connectivity_issue,
            quality_issues=quality_issues,
            spam=spam,
            call_score=call_score,
            comment=comment,
            incident=incident,
        )

    def fetch(self):
        """
        Fetch the AnnotationInstance

        :returns: The fetched AnnotationInstance
        :rtype: twilio.rest.insights.v1.call.annotation.AnnotationInstance
        """
        return self._proxy.fetch()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Insights.V1.AnnotationInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.version import Version
from twilio.rest.media.v1.media_processor import MediaProcessorList
from twilio.rest.media.v1.media_recording import MediaRecordingList
from twilio.rest.media.v1.player_streamer import PlayerStreamerList


class V1(Version):

    def __init__(self, domain):
        """
        Initialize the V1 version of Media

        :returns: V1 version of Media
        :rtype: twilio.rest.media.v1.V1.V1
        """
        super(V1, self).__init__(domain)
        self.version = 'v1'
        self._media_processor = None
        self._media_recording = None
        self._player_streamer = None

    @property
    def media_processor(self):
        """
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorList
        """
        if self._media_processor is None:
            self._media_processor = MediaProcessorList(self)
        return self._media_processor

    @property
    def media_recording(self):
        """
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingList
        """
        if self._media_recording is None:
            self._media_recording = MediaRecordingList(self)
        return self._media_recording

    @property
    def player_streamer(self):
        """
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerList
        """
        if self._player_streamer is None:
            self._player_streamer = PlayerStreamerList(self)
        return self._player_streamer

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Media.V1>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import serialize
from twilio.base import values
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class StreamMessageList(ListResource):

    def __init__(self, version, service_sid, stream_sid):
        """
        Initialize the StreamMessageList

        :param Version version: Version that contains the resource
        :param service_sid: The SID of the Sync Service that the resource is associated with
        :param stream_sid: The unique string that identifies the resource

        :returns: twilio.rest.sync.v1.service.sync_stream.stream_message.StreamMessageList
        :rtype: twilio.rest.sync.v1.service.sync_stream.stream_message.StreamMessageList
        """
        super(StreamMessageList, self).__init__(version)

        # Path Solution
        self._solution = {'service_sid': service_sid, 'stream_sid': stream_sid, }
        self._uri = '/Services/{service_sid}/Streams/{stream_sid}/Messages'.format(**self._solution)

    def create(self, data):
        """
        Create the StreamMessageInstance

        :param dict data: A JSON string that represents an arbitrary, schema-less object that makes up the Stream Message body

        :returns: The created StreamMessageInstance
        :rtype: twilio.rest.sync.v1.service.sync_stream.stream_message.StreamMessageInstance
        """
        data = values.of({'Data': serialize.object(data), })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return StreamMessageInstance(
            self._version,
            payload,
            service_sid=self._solution['service_sid'],
            stream_sid=self._solution['stream_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Sync.V1.StreamMessageList>'


class StreamMessagePage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the StreamMessagePage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param service_sid: The SID of the Sync Service that the resource is associated with
        :param stream_sid: The unique string that identifies the resource

        :returns: twilio.rest.sync.v1.service.sync_stream.stream_message.StreamMessagePage
        :rtype: twilio.rest.sync.v1.service.sync_stream.stream_message.StreamMessagePage
        """
        super(StreamMessagePage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of StreamMessageInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.sync.v1.service.sync_stream.stream_message.StreamMessageInstance
        :rtype: twilio.rest.sync.v1.service.sync_stream.stream_message.StreamMessageInstance
        """
        return StreamMessageInstance(
            self._version,
            payload,
            service_sid=self._solution['service_sid'],
            stream_sid=self._solution['stream_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Sync.V1.StreamMessagePage>'


class StreamMessageInstance(InstanceResource):

    def __init__(self, version, payload, service_sid, stream_sid):
        """
        Initialize the StreamMessageInstance

        :returns: twilio.rest.sync.v1.service.sync_stream.stream_message.StreamMessageInstance
        :rtype: twilio.rest.sync.v1.service.sync_stream.stream_message.StreamMessageInstance
        """
        super(StreamMessageInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {'sid': payload.get('sid'), 'data': payload.get('data'), }

        # Context
        self._context = None
        self._solution = {'service_sid': service_sid, 'stream_sid': stream_sid, }

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def data(self):
        """
        :returns: Stream Message body
        :rtype: dict
        """
        return self._properties['data']

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Sync.V1.StreamMessageInstance>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import serialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class InteractionChannelParticipantList(ListResource):

    def __init__(self, version, interaction_sid, channel_sid):
        """
        Initialize the InteractionChannelParticipantList

        :param Version version: Version that contains the resource
        :param interaction_sid: The Interaction Sid for this channel.
        :param channel_sid: The Channel Sid for this Participant.

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantList
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantList
        """
        super(InteractionChannelParticipantList, self).__init__(version)

        # Path Solution
        self._solution = {'interaction_sid': interaction_sid, 'channel_sid': channel_sid, }
        self._uri = '/Interactions/{interaction_sid}/Channels/{channel_sid}/Participants'.format(**self._solution)

    def create(self, type, media_properties):
        """
        Create the InteractionChannelParticipantInstance

        :param InteractionChannelParticipantInstance.Type type: Participant type.
        :param dict media_properties: JSON representing the Media Properties for the new Participant.

        :returns: The created InteractionChannelParticipantInstance
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantInstance
        """
        data = values.of({'Type': type, 'MediaProperties': serialize.object(media_properties), })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return InteractionChannelParticipantInstance(
            self._version,
            payload,
            interaction_sid=self._solution['interaction_sid'],
            channel_sid=self._solution['channel_sid'],
        )

    def stream(self, limit=None, page_size=None):
        """
        Streams InteractionChannelParticipantInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, limit=None, page_size=None):
        """
        Lists InteractionChannelParticipantInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantInstance]
        """
        return list(self.stream(limit=limit, page_size=page_size, ))

    def page(self, page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of InteractionChannelParticipantInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of InteractionChannelParticipantInstance
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantPage
        """
        data = values.of({'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return InteractionChannelParticipantPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of InteractionChannelParticipantInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of InteractionChannelParticipantInstance
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return InteractionChannelParticipantPage(self._version, response, self._solution)

    def get(self, sid):
        """
        Constructs a InteractionChannelParticipantContext

        :param sid: The unique string that identifies the resource

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantContext
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantContext
        """
        return InteractionChannelParticipantContext(
            self._version,
            interaction_sid=self._solution['interaction_sid'],
            channel_sid=self._solution['channel_sid'],
            sid=sid,
        )

    def __call__(self, sid):
        """
        Constructs a InteractionChannelParticipantContext

        :param sid: The unique string that identifies the resource

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantContext
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantContext
        """
        return InteractionChannelParticipantContext(
            self._version,
            interaction_sid=self._solution['interaction_sid'],
            channel_sid=self._solution['channel_sid'],
            sid=sid,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V1.InteractionChannelParticipantList>'


class InteractionChannelParticipantPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the InteractionChannelParticipantPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param interaction_sid: The Interaction Sid for this channel.
        :param channel_sid: The Channel Sid for this Participant.

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantPage
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantPage
        """
        super(InteractionChannelParticipantPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of InteractionChannelParticipantInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantInstance
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantInstance
        """
        return InteractionChannelParticipantInstance(
            self._version,
            payload,
            interaction_sid=self._solution['interaction_sid'],
            channel_sid=self._solution['channel_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V1.InteractionChannelParticipantPage>'


class InteractionChannelParticipantContext(InstanceContext):

    def __init__(self, version, interaction_sid, channel_sid, sid):
        """
        Initialize the InteractionChannelParticipantContext

        :param Version version: Version that contains the resource
        :param interaction_sid: The Interaction Sid for this channel.
        :param channel_sid: The Channel Sid for this Participant.
        :param sid: The unique string that identifies the resource

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantContext
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantContext
        """
        super(InteractionChannelParticipantContext, self).__init__(version)

        # Path Solution
        self._solution = {'interaction_sid': interaction_sid, 'channel_sid': channel_sid, 'sid': sid, }
        self._uri = '/Interactions/{interaction_sid}/Channels/{channel_sid}/Participants/{sid}'.format(**self._solution)

    def update(self, status):
        """
        Update the InteractionChannelParticipantInstance

        :param InteractionChannelParticipantInstance.Status status: The Participant's status.

        :returns: The updated InteractionChannelParticipantInstance
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantInstance
        """
        data = values.of({'Status': status, })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return InteractionChannelParticipantInstance(
            self._version,
            payload,
            interaction_sid=self._solution['interaction_sid'],
            channel_sid=self._solution['channel_sid'],
            sid=self._solution['sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.FlexApi.V1.InteractionChannelParticipantContext {}>'.format(context)


class InteractionChannelParticipantInstance(InstanceResource):

    class Status(object):
        CLOSED = "closed"
        WRAPUP = "wrapup"

    class Type(object):
        SUPERVISOR = "supervisor"
        CUSTOMER = "customer"
        EXTERNAL = "external"
        AGENT = "agent"
        UNKNOWN = "unknown"

    def __init__(self, version, payload, interaction_sid, channel_sid, sid=None):
        """
        Initialize the InteractionChannelParticipantInstance

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantInstance
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantInstance
        """
        super(InteractionChannelParticipantInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'type': payload.get('type'),
            'interaction_sid': payload.get('interaction_sid'),
            'channel_sid': payload.get('channel_sid'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {
            'interaction_sid': interaction_sid,
            'channel_sid': channel_sid,
            'sid': sid or self._properties['sid'],
        }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: InteractionChannelParticipantContext for this InteractionChannelParticipantInstance
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantContext
        """
        if self._context is None:
            self._context = InteractionChannelParticipantContext(
                self._version,
                interaction_sid=self._solution['interaction_sid'],
                channel_sid=self._solution['channel_sid'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def type(self):
        """
        :returns: Participant type.
        :rtype: InteractionChannelParticipantInstance.Type
        """
        return self._properties['type']

    @property
    def interaction_sid(self):
        """
        :returns: The Interaction Sid for this channel.
        :rtype: unicode
        """
        return self._properties['interaction_sid']

    @property
    def channel_sid(self):
        """
        :returns: The Channel Sid for this Participant.
        :rtype: unicode
        """
        return self._properties['channel_sid']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    def update(self, status):
        """
        Update the InteractionChannelParticipantInstance

        :param InteractionChannelParticipantInstance.Status status: The Participant's status.

        :returns: The updated InteractionChannelParticipantInstance
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantInstance
        """
        return self._proxy.update(status, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.FlexApi.V1.InteractionChannelParticipantInstance {}>'.format(context)



import base64
import hmac
from hashlib import sha1, sha256

from urllib.parse import urlparse, parse_qs


def compare(string1, string2):
    """Compare two strings while protecting against timing attacks

    :param str string1: the first string
    :param str string2: the second string

    :returns: True if the strings are equal, False if not
    :rtype: :obj:`bool`
    """
    if len(string1) != len(string2):
        return False
    result = True
    for c1, c2 in zip(string1, string2):
        result &= c1 == c2

    return result


def remove_port(uri):
    """Remove the port number from a URI

    :param uri: parsed URI that Twilio requested on your server

    :returns: full URI without a port number
    :rtype: str
    """
    if not uri.port:
        return uri.geturl()

    new_netloc = uri.netloc.split(':')[0]
    new_uri = uri._replace(netloc=new_netloc)

    return new_uri.geturl()


def add_port(uri):
    """Add the port number to a URI

    :param uri: parsed URI that Twilio requested on your server

    :returns: full URI with a port number
    :rtype: str
    """
    if uri.port:
        return uri.geturl()

    port = 443 if uri.scheme == "https" else 80
    new_netloc = uri.netloc + ":" + str(port)
    new_uri = uri._replace(netloc=new_netloc)

    return new_uri.geturl()


class RequestValidator(object):

    def __init__(self, token):
        self.token = token.encode("utf-8")

    def compute_signature(self, uri, params):
        """Compute the signature for a given request

        :param uri: full URI that Twilio requested on your server
        :param params: post vars that Twilio sent with the request

        :returns: The computed signature
        """
        s = uri
        if params:
            for param_name in sorted(set(params)):
                values = self.get_values(params, param_name)

                for value in sorted(set(values)):
                    s += param_name + value

        # compute signature and compare signatures
        mac = hmac.new(self.token, s.encode("utf-8"), sha1)
        computed = base64.b64encode(mac.digest())
        computed = computed.decode('utf-8')

        return computed.strip()

    def get_values(self, param_dict, param_name):
        try:
            # Support MultiDict used by Flask.
            return param_dict.getall(param_name)
        except AttributeError:
            try:
                # Support QueryDict used by Django.
                return param_dict.getlist(param_name)
            except AttributeError:
                # Fallback to a standard dict.
                return [param_dict[param_name]]

    def compute_hash(self, body):
        computed = sha256(body.encode("utf-8")).hexdigest()

        return computed.strip()

    def validate(self, uri, params, signature):
        """Validate a request from Twilio

        :param uri: full URI that Twilio requested on your server
        :param params: dictionary of POST variables or string of POST body for JSON requests
        :param signature: expected signature in HTTP X-Twilio-Signature header

        :returns: True if the request passes validation, False if not
        """
        if params is None:
            params = {}

        parsed_uri = urlparse(uri)
        uri_with_port = add_port(parsed_uri)
        uri_without_port = remove_port(parsed_uri)

        valid_body_hash = True  # May not receive body hash, so default succeed

        query = parse_qs(parsed_uri.query)
        if "bodySHA256" in query and isinstance(params, str):
            valid_body_hash = compare(self.compute_hash(params), query["bodySHA256"][0])
            params = {}

        #  check signature of uri with and without port,
        #  since sig generation on back end is inconsistent
        valid_signature = compare(self.compute_signature(uri_without_port, params), signature)
        valid_signature_with_port = compare(self.compute_signature(uri_with_port, params), signature)

        return valid_body_hash and (valid_signature or valid_signature_with_port)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class UserRolesList(ListResource):

    def __init__(self, version):
        """
        Initialize the UserRolesList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.flex_api.v1.user_roles.UserRolesList
        :rtype: twilio.rest.flex_api.v1.user_roles.UserRolesList
        """
        super(UserRolesList, self).__init__(version)

        # Path Solution
        self._solution = {}

    def get(self):
        """
        Constructs a UserRolesContext

        :returns: twilio.rest.flex_api.v1.user_roles.UserRolesContext
        :rtype: twilio.rest.flex_api.v1.user_roles.UserRolesContext
        """
        return UserRolesContext(self._version, )

    def __call__(self):
        """
        Constructs a UserRolesContext

        :returns: twilio.rest.flex_api.v1.user_roles.UserRolesContext
        :rtype: twilio.rest.flex_api.v1.user_roles.UserRolesContext
        """
        return UserRolesContext(self._version, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V1.UserRolesList>'


class UserRolesPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the UserRolesPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.flex_api.v1.user_roles.UserRolesPage
        :rtype: twilio.rest.flex_api.v1.user_roles.UserRolesPage
        """
        super(UserRolesPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of UserRolesInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.flex_api.v1.user_roles.UserRolesInstance
        :rtype: twilio.rest.flex_api.v1.user_roles.UserRolesInstance
        """
        return UserRolesInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V1.UserRolesPage>'


class UserRolesContext(InstanceContext):

    def __init__(self, version):
        """
        Initialize the UserRolesContext

        :param Version version: Version that contains the resource

        :returns: twilio.rest.flex_api.v1.user_roles.UserRolesContext
        :rtype: twilio.rest.flex_api.v1.user_roles.UserRolesContext
        """
        super(UserRolesContext, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/Accounts/UserRoles'.format(**self._solution)

    def fetch(self, token=values.unset):
        """
        Fetch the UserRolesInstance

        :param unicode token: The Token HTTP request header

        :returns: The fetched UserRolesInstance
        :rtype: twilio.rest.flex_api.v1.user_roles.UserRolesInstance
        """
        headers = values.of({'Token': token, })

        payload = self._version.fetch(method='GET', uri=self._uri, headers=headers, )

        return UserRolesInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.FlexApi.V1.UserRolesContext {}>'.format(context)


class UserRolesInstance(InstanceResource):

    def __init__(self, version, payload):
        """
        Initialize the UserRolesInstance

        :returns: twilio.rest.flex_api.v1.user_roles.UserRolesInstance
        :rtype: twilio.rest.flex_api.v1.user_roles.UserRolesInstance
        """
        super(UserRolesInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {'roles': payload.get('roles'), 'url': payload.get('url'), }

        # Context
        self._context = None
        self._solution = {}

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: UserRolesContext for this UserRolesInstance
        :rtype: twilio.rest.flex_api.v1.user_roles.UserRolesContext
        """
        if self._context is None:
            self._context = UserRolesContext(self._version, )
        return self._context

    @property
    def roles(self):
        """
        :returns: Flex Insights roles for the user
        :rtype: list[unicode]
        """
        return self._properties['roles']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self, token=values.unset):
        """
        Fetch the UserRolesInstance

        :param unicode token: The Token HTTP request header

        :returns: The fetched UserRolesInstance
        :rtype: twilio.rest.flex_api.v1.user_roles.UserRolesInstance
        """
        return self._proxy.fetch(token=token, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.FlexApi.V1.UserRolesInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class UserInfoList(ListResource):

    def __init__(self, version):
        """
        Initialize the UserInfoList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.oauth.v1.user_info.UserInfoList
        :rtype: twilio.rest.oauth.v1.user_info.UserInfoList
        """
        super(UserInfoList, self).__init__(version)

        # Path Solution
        self._solution = {}

    def get(self):
        """
        Constructs a UserInfoContext

        :returns: twilio.rest.oauth.v1.user_info.UserInfoContext
        :rtype: twilio.rest.oauth.v1.user_info.UserInfoContext
        """
        return UserInfoContext(self._version, )

    def __call__(self):
        """
        Constructs a UserInfoContext

        :returns: twilio.rest.oauth.v1.user_info.UserInfoContext
        :rtype: twilio.rest.oauth.v1.user_info.UserInfoContext
        """
        return UserInfoContext(self._version, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Oauth.V1.UserInfoList>'


class UserInfoPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the UserInfoPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.oauth.v1.user_info.UserInfoPage
        :rtype: twilio.rest.oauth.v1.user_info.UserInfoPage
        """
        super(UserInfoPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of UserInfoInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.oauth.v1.user_info.UserInfoInstance
        :rtype: twilio.rest.oauth.v1.user_info.UserInfoInstance
        """
        return UserInfoInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Oauth.V1.UserInfoPage>'


class UserInfoContext(InstanceContext):

    def __init__(self, version):
        """
        Initialize the UserInfoContext

        :param Version version: Version that contains the resource

        :returns: twilio.rest.oauth.v1.user_info.UserInfoContext
        :rtype: twilio.rest.oauth.v1.user_info.UserInfoContext
        """
        super(UserInfoContext, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/userinfo'.format(**self._solution)

    def fetch(self):
        """
        Fetch the UserInfoInstance

        :returns: The fetched UserInfoInstance
        :rtype: twilio.rest.oauth.v1.user_info.UserInfoInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return UserInfoInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Oauth.V1.UserInfoContext {}>'.format(context)


class UserInfoInstance(InstanceResource):

    def __init__(self, version, payload):
        """
        Initialize the UserInfoInstance

        :returns: twilio.rest.oauth.v1.user_info.UserInfoInstance
        :rtype: twilio.rest.oauth.v1.user_info.UserInfoInstance
        """
        super(UserInfoInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'user_sid': payload.get('user_sid'),
            'first_name': payload.get('first_name'),
            'last_name': payload.get('last_name'),
            'friendly_name': payload.get('friendly_name'),
            'email': payload.get('email'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {}

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: UserInfoContext for this UserInfoInstance
        :rtype: twilio.rest.oauth.v1.user_info.UserInfoContext
        """
        if self._context is None:
            self._context = UserInfoContext(self._version, )
        return self._context

    @property
    def user_sid(self):
        """
        :returns: The user sid
        :rtype: unicode
        """
        return self._properties['user_sid']

    @property
    def first_name(self):
        """
        :returns: The first name of the end-user
        :rtype: unicode
        """
        return self._properties['first_name']

    @property
    def last_name(self):
        """
        :returns: The last name of the end-user
        :rtype: unicode
        """
        return self._properties['last_name']

    @property
    def friendly_name(self):
        """
        :returns: The friendly name of the end-user
        :rtype: unicode
        """
        return self._properties['friendly_name']

    @property
    def email(self):
        """
        :returns: The end-user's preferred email address
        :rtype: unicode
        """
        return self._properties['email']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self):
        """
        Fetch the UserInfoInstance

        :returns: The fetched UserInfoInstance
        :rtype: twilio.rest.oauth.v1.user_info.UserInfoInstance
        """
        return self._proxy.fetch()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Oauth.V1.UserInfoInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.domain import Domain
from twilio.rest.routes.v2 import V2


class Routes(Domain):

    def __init__(self, twilio):
        """
        Initialize the Routes Domain

        :returns: Domain for Routes
        :rtype: twilio.rest.routes.Routes
        """
        super(Routes, self).__init__(twilio)

        self.base_url = 'https://routes.twilio.com'

        # Versions
        self._v2 = None

    @property
    def v2(self):
        """
        :returns: Version v2 of routes
        :rtype: twilio.rest.routes.v2.V2
        """
        if self._v2 is None:
            self._v2 = V2(self)
        return self._v2

    @property
    def phone_numbers(self):
        """
        :rtype: twilio.rest.routes.v2.phone_number.PhoneNumberList
        """
        return self.v2.phone_numbers

    @property
    def sip_domains(self):
        """
        :rtype: twilio.rest.routes.v2.sip_domain.SipDomainList
        """
        return self.v2.sip_domains

    @property
    def trunks(self):
        """
        :rtype: twilio.rest.routes.v2.trunk.TrunkList
        """
        return self.v2.trunks

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Routes>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.domain import Domain
from twilio.rest.messaging.v1 import V1


class Messaging(Domain):

    def __init__(self, twilio):
        """
        Initialize the Messaging Domain

        :returns: Domain for Messaging
        :rtype: twilio.rest.messaging.Messaging
        """
        super(Messaging, self).__init__(twilio)

        self.base_url = 'https://messaging.twilio.com'

        # Versions
        self._v1 = None

    @property
    def v1(self):
        """
        :returns: Version v1 of messaging
        :rtype: twilio.rest.messaging.v1.V1
        """
        if self._v1 is None:
            self._v1 = V1(self)
        return self._v1

    @property
    def brand_registrations(self):
        """
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationList
        """
        return self.v1.brand_registrations

    @property
    def deactivations(self):
        """
        :rtype: twilio.rest.messaging.v1.deactivation.DeactivationsList
        """
        return self.v1.deactivations

    @property
    def external_campaign(self):
        """
        :rtype: twilio.rest.messaging.v1.external_campaign.ExternalCampaignList
        """
        return self.v1.external_campaign

    @property
    def services(self):
        """
        :rtype: twilio.rest.messaging.v1.service.ServiceList
        """
        return self.v1.services

    @property
    def tollfree_verifications(self):
        """
        :rtype: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationList
        """
        return self.v1.tollfree_verifications

    @property
    def usecases(self):
        """
        :rtype: twilio.rest.messaging.v1.usecase.UsecaseList
        """
        return self.v1.usecases

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Messaging>'
